# -*- coding: utf-8 -*-
"""
    pod2gen.warnings
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This file contains pod2gen-specific warnings.
    They can be imported directly from ``pod2gen``.

    :copyright: 2019, Thorben Dahl <thorben@sjostrom.no>
        2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""


class Pod2genWarning(UserWarning):
    """
    Superclass for all warnings defined by pod2gen.
    """

    pass


class NotRecommendedWarning(Pod2genWarning):
    """
    Warns against behaviour or usage which is usually discouraged. However,
    there may exist exceptions where there is no better way.
    """

    pass


class LegacyCategoryWarning(Pod2genWarning):
    """
    Indicates that the category created is an old category. It will still be
    accepted by Apple Podcasts, but it would be wise to use the new categories
    since they may have more relevant options for your podcast.

    .. seealso::

       `What's New: Enhanced Apple Podcasts Categories <https://itunespartner.apple.com/podcasts/whats-new/100002564>`_
          Consequences of using old categories.

       `Podcasts Connect Help: Apple Podcasts categories <https://help.apple.com/itc/podcasts_connect/#/itc9267a2f12>`_
          Up-to-date list of available categories.

       `Podnews: New and changed Apple Podcasts categories <https://podnews.net/article/apple-changed-podcast-categories-2019>`_
          List of changes between the old and the new categories.
    """

    pass


class NotSupportedByItunesWarning(Pod2genWarning):
    """
    Indicates that pod2gen is used in a way that may not be compatible with Apple
    Podcasts (previously known as iTunes).

    In some cases, this may be because pod2gen has not been kept up-to-date with
    new features which Apple Podcasts has added support for. Please add an issue
    if that is the case!
    """

    pass


class LockedTagCannotBeSet(Pod2genWarning):
    """
    Indicates that the tag ``<podcast:locked>`` cannot be generated.

    The Locked tag requires the podcast owner email to be defined
    """

    pass


class UnknownPersonGroup(Pod2genWarning):
    """
    Indicates that the person group for the tag ```<podcast:person>``` is not
    within the Podcast Taxonomy Project list.

    Check the below link for the full podcast taxonomy:
    https://github.com/Podcastindex-org/podcast-namespace/blob/main/taxonomy.json
    """

    pass


class UnknownPersonRole(Pod2genWarning):
    """
    Indicates that the person role for the tag ``<podcast:person>`` is not
    within the Podcast Taxonomy Project list.

    Check the below link for the full podcast taxonomy:
    https://github.com/Podcastindex-org/podcast-namespace/blob/main/taxonomy.json
    """

    pass


class UnknownPersonGroupRoleTuple(Pod2genWarning):
    """
    Indicates that the person role and group for the tag ``<podcast:person>`` does
    not match according to the Podcast Taxonomy Project list.

    Check the below link for the full podcast taxonomy:
    https://github.com/Podcastindex-org/podcast-namespace/blob/main/taxonomy.json
    """

    pass



# -*- coding: utf-8 -*-
"""
    pod2gen.funding
    ~~~~~~~~~~~~~~~

    This module contains Funding, which represents a funding element.

    :copyright: 2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import warnings
from urllib.parse import urlparse

import validators

from pod2gen.warnings import NotSupportedByItunesWarning


class Funding(object):
    """Class representing a Podcast funding page.

    By using this class, you can be sure that the funding object is formatted correctly
    and will be correctly used by the :class:`Podcast` class for feed generation.

    :class:`.Funding` objects can be attached to :class:`.Podcast` objects
    using this method. Fundings are stored in :attr:`.Podcast.fundings`.

    See `podcast-namespace Funding
    <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#funding>`_
    for an overview of the Funding tag

    Example::

        >>> from pod2gen import Funding, Podcast
        >>> f = Funding("Please support me!", "https://examples.com/donation_link.html")
        >>> f.text
        'Please support me!'
        >>> f.url
        'https://examples.com/donation_link.html'
        >>> f.text = "Support us if you like the show!"
        >>> f.text
        'Support us if you like the show!'
        >>> f.url = "https://examples.com/another_donation_link.html"
        >>> f.url
        'https://examples.com/another_donation_link.html'
        >>> p = Podcast()
        >>> p.add_funding(f)
    """

    def __init__(self, text, url):
        """Create new Funding object. See the class description of
        :class:´~pod2gen.funding.Funding`.

        :param text: A funding message (e.g. "Support the show!").
        :type text: str
        :param url: Donation/funding link (Must be a valid url).
        :type url: str
        """

        self.__text = None
        self.__url = None

        self.text = text
        self.url = url

    @property
    def text(self):
        """A message to encourage people donation and support.
        The message should not exceed 128 characters.

        Check `podcast-namespace Funding node value
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#node-value-1>`_
        for more details.

        :type: :obj:`str`
        """
        return self.__text

    @text.setter
    def text(self, text):
        if not text:
            raise ValueError("Funding text cannot be empty or None")

        if not isinstance(text, str):
            raise ValueError("Funding text must be a valid string")

        if len(text) > 128:
            raise ValueError("Funding text must not exceed 128 characters")

        self.__text = text

    @property
    def url(self):
        """The donation/link for a podcast.

        Check `podcast-namespace Funding url attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-2>`_
        for more details.

        :type: :obj:`str`
        """
        return self.__url

    @url.setter
    def url(self, url):
        if not url:
            raise ValueError("url cannot be empty or None")

        if validators.url(url) is not True:
            raise ValueError("Not a valid url: %s" % url)

        parsed_url = urlparse(url)
        if parsed_url.scheme not in ("http", "https"):
            warnings.warn(
                "URL scheme %s is not supported by iTunes. Make sure "
                "you use absolute URLs and HTTP or HTTPS." % parsed_url.scheme,
                NotSupportedByItunesWarning,
                stacklevel=2,
            )
        self.__url = url

    def __repr__(self):
        return "Funding(text=%s, url=%s)" % (self.text, self.url)



# -*- coding: utf-8 -*-
"""
    pod2gen.license
    ~~~~~~~~~~~~~~~

    This module contains License class that represents a license that is applied
    to the audio/video content of a single episode, or the audio/video of the
    podcast as a whole.

    :copyright: 2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import warnings
from urllib.parse import urlparse

import validators

from pod2gen.warnings import NotSupportedByItunesWarning


class License(object):
    """Class representing a license object that is applied to the audio/video content
    of a single episode, or the audio/video of the podcast as a whole.

    Custom licenses must always include a url attribute. Implementors are encouraged to
    read the license tag companion document for a more complete picture of what this tag
    is intended to accomplish.

    By using this class, you can be sure that the license object is formatted correctly
    and will be correctly used by the associated :class:`~.pod2gen.Podcast` or
    :class:`~.pod2gen.Episode` object.

    See `podcast-namespace License
    <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#license>`_
    for an overview of the License tag.

    Example::

        >>> from pod2gen import License
        >>> l = License("my-podcast-license-v1", "https://example.org/mypodcastlicense/full.pdf")
        >>> l.identifier
        'my-podcast-license-v1'
        >>> l.url
        'https://example.org/mypodcastlicense/full.pdf'
        >>> l.identifier = "my-podcast-license-v2"
        >>> l.identifier
        'my-podcast-license-v2'
        >>> l.url = "https://example.org/mypodcastlicense/full_2.pdf"
        >>> l.url
        'https://example.org/mypodcastlicense/full_2.pdf'
    """

    def __init__(self, identifier, url=None):
        """Create new License object. See the class description of
        :class:´~pod2gen.license.License`.

        :param identifier: A license "identifier" defined in the SPDX License List file if
            the license being used is a well-known, public license. Or, if it is a
            custom license, it must be a free form abbreviation of the name of the
            license as you reference it publicly.
        :type identifier: str
        :param url: (Optional) a url that points to the full, legal language of the
            license being referenced.
        :type url: str
        """

        self.__identifier = None
        self.__url = None

        self.identifier = identifier
        self.url = url

    @property
    def identifier(self):
        """A license "identifier" defined in the
        `SPDX License List <https://spdx.org/licenses/>`_ file if
        the license being used is a well-known, public license. If it is a
        custom license, it must be a free form abbreviation of the license
        name as you reference it publicly.

        Check `podcast-namespace License node value
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#node-value-8>`_
        for more details.

        :type: :obj:`str`
        """
        return self.__identifier

    @identifier.setter
    def identifier(self, identifier):
        if not identifier:
            raise ValueError("License identifier cannot be empty or None")

        if not isinstance(identifier, str):
            raise ValueError("License identifier must be a valid string")

        self.__identifier = identifier

    @property
    def url(self):
        """Url that points to the full, legal language of the license being referenced.

        Check `podcast-namespace License url attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-10>`_
        for more details

        :type: :obj:`str` or :obj:`None`
        """
        return self.__url

    @url.setter
    def url(self, url):
        if url:
            if validators.url(url) is not True:
                raise ValueError("Not a valid url: %s" % url)

            parsed_url = urlparse(url)
            if parsed_url.scheme not in ("http", "https"):
                warnings.warn(
                    "URL scheme %s is not supported by iTunes. Make sure "
                    "you use absolute URLs and HTTP or HTTPS." % parsed_url.scheme,
                    NotSupportedByItunesWarning,
                    stacklevel=2,
                )
            self.__url = url

        else:
            self.__url = None

    def __repr__(self):
        if self.url:
            return "License(identifier=%s, url=%s)" % (self.identifier, self.url)
        return "License(identifier=%s)" % self.identifier



# -*- coding: utf-8 -*-
"""
    pod2gen.person
    ~~~~~~~~~~~~~

    This file contains the Person class, which is used to represent a person or
    an entity.

    :copyright: 2016, Thorben Dahl <thorben@sjostrom.no>
        2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import warnings
from urllib.parse import urlparse

import validators
from lxml import etree

from pod2gen.warnings import (
    NotSupportedByItunesWarning,
    UnknownPersonGroup,
    UnknownPersonGroupRoleTuple,
    UnknownPersonRole,
)


class Person(object):
    """Data-oriented class representing a single person or entity.

    A Person can represent both real persons and less personal entities like
    organizations. Example::

        >>> p.authors = [Person("Example Radio", "mail@example.org")]

    The RSS Namespace Extension for Podcasting adds a ``<podcast:person>``
    tag for episodes and podcasts. :class:`~pod2gen.Person` can be appended
    to :class:`Episode.persons <pod2gen.Episode.persons>` using
    :meth:`Episode.add_person() <pod2gen.Episode.add_person>` or
    to :class:`Podcast.persons <pod2gen.Podcast.persons>` using
    :meth:`Podcast.add_person() <pod2gen.Podcast.add_person>`.

        >>> director = Person(name="Slim", group="Creative Direction", role="Director")
        >>> p.add_person(director)
        >>> guest = Person(email="mslimbeji@gmail.com", group="Cast", role="Guest")
        >>> my_episode.add_person(guest)

    .. note::

        At any time, one of name or email must be present.
        Both cannot be None or empty at the same time.

    .. warning::

        **Any names and email addresses** you put into a Person object will
        eventually be included and **published** together with
        the feed. If you want to keep a name or email address private, then you
        must make sure it isn't used in a Person object (or to be precise: that
        the Person object with the name or email address isn't used in any
        Podcast or Episode.)

    Example of use::

        >>> from pod2gen import Person, Podcast, Episode
        >>> Person("John Doe")
        Person(name=John Doe, email=None)
        >>> Person(email="johndoe@example.org")
        Person(name=None, email=johndoe@example.org)
        >>> Person()
        ValueError: You must provide either a name or an email address.
        >>> p = Person(name="Slim Beji", href="https://github.com/SlimBeji" img="https://examples.com/slimbeji.jpg")
        >>> p.name
        'Slim Beji'
        >>> p.href
        'https://github.com/SlimBeji'
        >>> p.img
        'https://examples.com/slimbeji.jpg'
        >>> p.img = "https://examples.com/slimbeji_2.png"
        >>> p.img
        'https://examples.com/slimbeji_2.png'
        >>> my_podcast = Podcast()
        >>> my_podcast.add_person(p)
        >>> my_episode = Episode
        >>> my_episode.add_person(p)

    .. seealso::
       :ref:`pod2gen.Person-guide`
          for a more gentle introduction.

    """

    _role_group_taxonomy_mapping = {
        "creative direction": [
            "director",
            "assistant director",
            "executive producer",
            "senior producer",
            "producer",
            "associate producer",
            "development producer",
            "creative director",
        ],
        "cast": [
            "host",
            "co-host",
            "guest host",
            "guest",
            "voice actor",
            "narrator",
            "announcer",
            "reporter",
        ],
        "writing": [
            "author",
            "editorial director",
            "co-writer",
            "writer",
            "songwriter",
            "guest writer",
            "story editor",
            "managing editor",
            "script editor",
            "script coordinator",
            "researcher",
            "editor",
            "fact checker",
            "translator",
            "transcriber",
            "logger",
        ],
        "audio production": [
            "studio coordinator",
            "technical director",
            "technical manager",
            "audio engineer",
            "remote recording engineer",
            "post production engineer",
        ],
        "audio post-production": [
            "audio editor",
            "sound designer",
            "foley artist",
            "composer",
            "theme music",
            "music production",
            "music contributor",
        ],
        "administration": [
            "production coordinator",
            "booking coordinator",
            "production assistant",
            "content manager",
            "marketing manager",
            "sales representative",
            "sales manager",
        ],
        "visuals": [
            "graphic designer",
            "cover art designer",
        ],
        "community": [
            "social media manager",
        ],
        "misc.": [
            "consultant",
            "intern",
        ],
        "video production": [
            "camera operator",
            "lighting designer",
            "camera grip",
            "assistant camera",
        ],
        "video post-production": [
            "editor",
            "assistant editor",
        ],
    }

    _available_role_taxonomy = [
        item for sublist in _role_group_taxonomy_mapping.values() for item in sublist
    ]

    def __init__(
        self, name=None, email=None, group=None, role=None, img=None, href=None
    ):
        """Create new person with a name, email or both.

        You don't need to provide both a name and an email, but you must
        provide one of them.

        :param name: This person's name.
        :type name: str or None
        :param email: This person's email address. The address it made public
            when the feed is published, so be careful about adding a personal
            email address here. The spambots are always on lookout!
        :type email: str or None
        :param group: used to identify what role the person serves on the show or episode.
        :type group: str or None
        :param role: this should be a reference to an official group within the Podcast
            Taxonomy Project list. See the below link for the full taxonomy
            https://github.com/Podcastindex-org/podcast-namespace/blob/main/taxonomy.json
        :type role: str or None
        :param img: This is the url of a picture or avatar of the person.
        :type img: str or None
        :param href: The url to a relevant resource of information about the person,
            such as a homepage or third-party profile platform.
        :type href: str or None

        """
        if not self._is_valid(name, email):
            raise ValueError("You must provide either a name or an email " "address.")

        self.__name = name
        self.__email = email
        self.__group = None
        self.__role = None
        self.__img = None
        self.__href = None

        self.group = group
        self.role = role
        self.img = img
        self.href = href

    def rss_entry(self):
        """Create an RSS ``<podcast:person>`` tag using lxml's etree and return it.

        This is primarily used by the :class:`~pod2gen.Podcast` class and the
        :class:`~pod2gen.Episode` class when generating RSS feeds as the Person
        tag parent is either the channel (podcast) or item (episode)

        :RSS: ``<podcast:person>``

        :returns: :class:`lxml.etree.Element`
        """
        PODCAST_NS = "https://podcastindex.org/namespace/1.0"
        entry = etree.Element("{%s}person" % PODCAST_NS)

        if self.name:
            entry.text = self.name
        else:
            entry.text = self.email

        if self.role:
            entry.attrib["role"] = self.role

        if self.group:
            entry.attrib["group"] = self.group

        if self.img:
            entry.attrib["img"] = self.img

        if self.href:
            entry.attrib["href"] = self.href

        if self.role and self.group:
            if self.role not in self._role_group_taxonomy_mapping.get(self.group, []):
                warnings.warn(
                    "role %s does not belong to the group %s according "
                    "to the podcast taxonomy project." % (self.role, self.group),
                    UnknownPersonGroupRoleTuple,
                    stacklevel=2,
                )

        return entry

    def _is_valid(self, name, email):
        """Check whether one of name and email are usable."""
        return name or email

    @property
    def name(self):
        """This person's name.

        :type: :obj:`str`
        """
        return self.__name

    @name.setter
    def name(self, new_name):
        if not self._is_valid(new_name, self.email):
            raise ValueError(
                "The name or email must be present at any time, "
                'cannot set name to "%s" as long as email is '
                '"%s"' % (new_name, self.email)
            )

        if new_name is not None:
            if len(new_name) > 128:
                raise ValueError("The name must not exceed 128 characters.")

        self.__name = new_name

    @property
    def email(self):
        """This person's public email address.

        :type: :obj:`str`
        """
        return self.__email

    @email.setter
    def email(self, new_email):
        if not self._is_valid(self.name, new_email):
            raise ValueError(
                "The name or email must be present at any time, "
                'cannot set email to "%s" as long as name is '
                '"%s"' % (new_email, self.name)
            )
        self.__email = new_email

    @property
    def group(self):
        """Used to identify what group of roles the person belongs to on the
        show or episode. This should be a reference to an official group within the
        `Podcast Taxonomy Project <https://podcasttaxonomy.com/home>`_ list.

        Check this link for the full Podcast Taxonomy `example
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/taxonomy.json>`_.

        See `podcast-namespace Person attributes
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-5>`_
        for more details about the group attribute.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__group

    @group.setter
    def group(self, group):
        if group:
            if not isinstance(group, str):
                raise ValueError("group must be a valid string")

            group = group.lower()
            if group not in self._role_group_taxonomy_mapping:
                warnings.warn(
                    "The group %s is not part of the official podcast taxonomy" % group,
                    UnknownPersonGroup,
                    stacklevel=2,
                )

            self.__group = group

        else:
            self.__group = None

    @property
    def role(self):
        """Used to identify what role the person serves on the
        show or episode. This should be a reference to an official role within the
        `Podcast Taxonomy Project <https://podcasttaxonomy.com/home>`_ list.

        Check this link for the full Podcast Taxonomy `example
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/taxonomy.json>`_.

        See `podcast-namespace Person attributes
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-5>`_
        for more details about the role attribute.

        :type: :obj:`str` or :obj:`None``
        """
        return self.__role

    @role.setter
    def role(self, role):
        if role:
            if not isinstance(role, str):
                raise ValueError("role must be a valid string")

            role = role.lower()
            if role not in self._available_role_taxonomy:
                warnings.warn(
                    "The role %s is not part of the official podcast taxonomy" % role,
                    UnknownPersonRole,
                    stacklevel=2,
                )

            self.__role = role

        else:
            self.__role = None

    @property
    def img(self):
        """This is the url of a picture or avatar of the person.

        See `podcast-namespace Person attributes
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-5>`_
        for more details about the img attribute.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__img

    @img.setter
    def img(self, img):
        if img is not None:
            if validators.url(img) is not True:
                raise ValueError("Not a valid url: %s" % img)

            parsed_url = urlparse(img)
            if parsed_url.scheme not in ("http", "https"):
                warnings.warn(
                    "URL scheme %s is not supported by iTunes. Make sure "
                    "you use absolute URLs and HTTP or HTTPS." % parsed_url.scheme,
                    NotSupportedByItunesWarning,
                    stacklevel=2,
                )
            self.__img = img
        else:
            self.__img = None

    @property
    def href(self):
        """The url to a relevant resource of information about the person,
        such as a homepage or third-party profile platform.

        See `podcast-namespace Person attributes
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-5>`_
        for more details about the href attribute.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__href

    @href.setter
    def href(self, href):
        if href is not None:
            if validators.url(href) is not True:
                raise ValueError("Not a valid url: %s" % href)

            parsed_url = urlparse(href)
            if parsed_url.scheme not in ("http", "https"):
                warnings.warn(
                    "URL scheme %s is not supported by iTunes. Make sure "
                    "you use absolute URLs and HTTP or HTTPS." % parsed_url.scheme,
                    NotSupportedByItunesWarning,
                    stacklevel=2,
                )
            self.__href = href
        else:
            self.__href = None

    def __str__(self):
        if self.email is None:
            return self.name
        elif self.name is None:
            return self.email
        else:
            return "%s (%s)" % (self.email, self.name)

    def __repr__(self):
        return "Person(name=%s, email=%s)" % (self.name, self.email)



# -*- coding: utf-8 -*-
"""
    pod2gen.alternate_media
    ~~~~~~~~~~~~

    This file contains the AlternateMedia class which is the implementation of the RSS
    element podcast:alternateEnclosure representing an alternative to the RSS element
    enclosure which is implemented in the pod2gen.media.Media class

    :copyright: 2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import pycountry
import validators
from lxml import etree


class AlternateMedia(object):
    """Data-oriented class representing several urls pointing to a media file.

    The AlternateMedia class represents the podcast:alternateEnclosure RSS element and is
    an alternative to the :class:`~pod2gen.Media` class representing enclosure RSS element.

    This element is meant to provide different versions of, or companion media to the
    main ``<enclosure>`` file. This could be an audio only version of a video podcast to
    allow apps to switch back and forth between audio/video, lower (or higher) bitrate
    versions for bandwidth constrained areas, alternative codecs for different device
    platforms, alternate URI schemes and download types such as IPFS or WebTorrent,
    commentary tracks or supporting source clips, etc.

        >>> from pod2gen import AlternateMedia
        >>> am = AlternateMedia(
        ...     "audio/mp4",
        ...     43200000,
        ...     bitrate=128000,
        ...     height=1080,
        ...     lang="en-US",
        ...     title="Standard",
        ...     rel="Off stage",
        ...     codecs="mp4a.40.2",
        ...     default=False,
        ...     encryption="sri",
        ...     signature="sha384-ExVqijgYHm15PqQqdXfW95x+Rs6C+d6E/ICxyQOeFevnxNLR/wtJNrNYTjIysUBo",
        ... )
        >>>


    In the `RSS Namespace Extension for Podcasting <https://podcastindex.org/namespace/1.0>`_,
    ``<podcast:alternateEnclosure>`` has two type of child tags:

    * ``<podcast:source>``
    * ``<podcast:integrity>``

    **Source tag**

    ``<podcast:source>`` defines a uri location for a media file. It is meant to be used
    as a child of the ``<podcast:alternateEnclosure>`` element, thus
    :class:`~pod2gen.AlternateMedia` must have at least one source added otherwise no
    ``<podcast:alternateEnclosure>``.

        >>> # Adding a file without specifying the content type
        >>> am.add_source("https://example.com/file-0.mp3")
        >>> # Adding a file, then specifying the content type later on
        >>> am.add_source("ipfs://QmdwGqd3d2gFPGeJNLLCshdiPert45fMu84552Y4XHTy4y")
        >>> am.edit_source_content(
        ...     "ipfs://QmdwGqd3d2gFPGeJNLLCshdiPert45fMu84552Y4XHTy4y",
        ...     "audio/mpeg",
        ... )
        >>> # Adding a file with a specified content type
        >>> am.add_source("https://example.com/file-0.torrent", "application/x-bittorrent")
        >>> # Deleting a source by its uri
        >>> am.delete_source("ipfs://QmdwGqd3d2gFPGeJNLLCshdiPert45fMu84552Y4XHTy4y")

    .. seealso::
       :ref:`pod2gen.Episode-alternateEnclosure-sources`
          for more details about Alternate enclosure sources.

    **Integrity tag**

    ``<podcast:integrity>`` defines a method of verifying integrity of the media
    given either an SRI-compliant integrity string (preferred) or a base64 encoded
    PGP signature. This tag is optional within a ``<podcast:alternateEnclosure>`` element.
    It allows to ensure that the file has not been tampered with.

    ``<podcast:integrity>`` is generated for a group of identical media file when the parent
    :class:`~pod2gen.AlternateMedia` object has a defined :obj:`~pod2gen.AlternateMedia.encryption`
    and :obj:`~pod2gen.AlternateMedia.signature` values.

        >>> am.encryption = "sri"
        >>> am.signature = "sha384-ExVqijgYHm15PqQqdXfW95x+Rs6C+d6E/ICxyQOeFevnxNLR/wtJNrNYTjIysUBo"

    .. seealso::
       :ref:`pod2gen.Episode-alternateEnclosure-integrity`
          for more details about Alternate enclosure integrity tag.

    """

    def __init__(
        self,
        type,
        length,
        bitrate=None,
        height=None,
        lang=None,
        title=None,
        rel=None,
        codecs=None,
        default=None,
        encryption=None,
        signature=None,
    ):

        self.__type = None
        self.__length = None
        self.__bitrate = None
        self.__height = None
        self.__lang = None
        self.__title = None
        self.__rel = None
        self.__codecs = None
        self.__default = None
        self.__encryption = None
        self.__signature = None
        self.__sources = {}

        self.type = type
        self.length = length
        self.bitrate = bitrate
        self.height = height
        self.lang = lang
        self.title = title
        self.rel = rel
        self.codecs = codecs
        self.default = default
        self.encryption = encryption
        self.signature = signature

    def rss_entry(self):
        """Create an RSS ``<podcast:alternateEnclosure>`` using lxml's etree and return it.

        This element is meant to provide different versions of, or companion media to
        the main ``<enclosure>`` file. This could be an audio only version of a video podcast
        to allow apps to switch back and forth between audio/video, lower (or higher)
        bitrate versions for bandwidth constrained areas, alternative codecs for different
        device platforms, alternate URI schemes and download types such as IPFS or WebTorrent,
        commentary tracks or supporting source clips, etc.

        This is primarily used by :class:`~.pod2gen.Episode` for building
        the RSS feed.

        :returns: :class:`lxml.etree.Element`
        :RSS: ``<podcast:alternateEnclosure>``
        """
        PODCAST_NS = "https://podcastindex.org/namespace/1.0"

        if self.__sources:
            element = etree.Element("{%s}alternateEnclosure" % PODCAST_NS)

            element.attrib["type"] = self.__type
            element.attrib["length"] = str(self.__length)

            if self.__bitrate:
                element.attrib["bitrate"] = str(self.__bitrate)

            if self.__height:
                element.attrib["height"] = str(self.__height)

            if self.__lang:
                element.attrib["lang"] = self.__lang

            if self.__title:
                element.attrib["title"] = self.__title

            if self.__rel:
                element.attrib["rel"] = self.__rel

            if self.__codecs:
                element.attrib["codecs"] = self.__codecs

            if self.__default is not None:
                if self.__default:
                    element.attrib["default"] = "true"
                else:
                    element.attrib["default"] = "false"

            for uri, content_type in self.__sources.items():
                source = etree.Element("{%s}source" % PODCAST_NS)
                source.attrib["uri"] = uri
                if content_type:
                    source.attrib["contentType"] = content_type
                element.append(source)

            if self.__encryption and self.__signature:
                integrity = etree.Element("{%s}integrity" % PODCAST_NS)
                integrity.attrib["type"] = self.__encryption
                integrity.attrib["value"] = self.__signature
                element.append(integrity)

            return element

        else:
            return None

    @property
    def type(self):
        """The MIME type of the files within the alternateEnclosure.

        See https://en.wikipedia.org/wiki/Media_type for an introduction.

        :type: :obj:`str`
        """
        return self.__type

    @type.setter
    def type(self, type):
        if not type:
            raise ValueError(
                "type property of SuperMedia objects cannot be empty or None"
            )

        if not isinstance(type, str):
            raise TypeError("File mime type must be a valid string")
        self.__type = type

    @property
    def length(self):
        """The length of the file bytes.

        Check `podcast-namespace Alternate Enclosure length attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`int`
        """
        return self.__length

    @length.setter
    def length(self, length):
        if not length:
            raise ValueError(
                "length property of SuperMedia objects cannot be empty or None"
            )

        try:
            length = int(length)
        except ValueError:
            raise TypeError(
                "length must be an integer. %r could not be converted into integer."
                % length
            )

        self.__length = length

    @property
    def bitrate(self):
        """The Encoding bitrate of media asset.

        Check `podcast-namespace Alternate Enclosure bitrate attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`float` or :obj:`None`
        """
        return self.__bitrate

    @bitrate.setter
    def bitrate(self, bitrate):
        if bitrate:
            try:
                bitrate = float(bitrate)
            except ValueError:
                raise TypeError(
                    "bitrate must be a float. %r could not be converted into float."
                    % bitrate
                )

            self.__bitrate = bitrate

        else:
            self.__bitrate = None

    @property
    def height(self):
        """Height of the media asset for video formats.

        Check `podcast-namespace Alternate Enclosure height attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`int` or :obj:`None`
        """
        return self.__height

    @height.setter
    def height(self, height):
        if height:
            try:
                height = int(height)
            except ValueError:
                raise TypeError(
                    "height must be an integer. %r could not be converted into integer."
                    % height
                )

            self.__height = height

        else:
            self.__height = None

    @property
    def lang(self):
        """An `IETF language tag (BCP 47) code <https://en.wikipedia.org/wiki/IETF_language_tag>`_
        identifying the language of the media files.

        Check `podcast-namespace Alternate Enclosure lang attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__lang

    @lang.setter
    def lang(self, lang):
        if lang:
            language_country = lang.split("-")
            language = language_country[0]
            if len(language_country) > 1:
                country = language_country[1]
            else:
                country = None

            try:
                language = pycountry.languages.get(alpha_2=language).alpha_2
            except Exception as e:
                raise ValueError("Language code %r is unknown" % language)

            if country:
                try:
                    country = pycountry.countries.lookup(country).alpha_2
                    self.__lang = language + "-" + country
                except:
                    raise ValueError("Country code %r is unknown" % country)
            else:
                self.__lang = language

        else:
            self.__lang = None

    @property
    def title(self):
        """A human-readable string identifying the name of the media asset.

        Check `podcast-namespace Alternate Enclosure title attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__title

    @title.setter
    def title(self, title):
        if title:
            if not isinstance(title, str):
                raise ValueError("Title field must be a valid string")

            if len(title) > 32:
                raise ValueError("Title field must not exceed 32 characters")

            self.__title = title

        else:
            self.__title = None

    @property
    def rel(self):
        """Provides a method of offering and/or grouping together different media elements.

        Check `podcast-namespace Alternate Enclosure rel attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__rel

    @rel.setter
    def rel(self, rel):
        if rel:
            if not isinstance(rel, str):
                raise ValueError("rel field must be a valid string")

            if len(rel) > 32:
                raise ValueError("rel field must not exceed 32 characters")

            self.__rel = rel

        else:
            self.__rel = None

    @property
    def codecs(self):
        """An `RFC 6381 string <https://datatracker.ietf.org/doc/html/rfc6381>`_
        specifying the codecs available in this media.

        Check `podcast-namespace Alternate Enclosure codecs attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__codecs

    @codecs.setter
    def codecs(self, codecs):
        if codecs:
            if not isinstance(codecs, str):
                raise ValueError("codecs field must be a valid string")

            self.__codecs = codecs

        else:
            self.__codecs = None

    @property
    def default(self):
        """Boolean specifying whether or not the given media is the same as the episode
        file in the Media object. If :obj:`True`, the given media should be considered the
        default one for the episode.

        Check `podcast-namespace Alternate Enclosure default attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-11>`_
        for more details.

        :type: :obj:`bool`
        """
        return self.__default

    @default.setter
    def default(self, default):
        if default is not None:
            if isinstance(default, bool):
                self.__default = default
            else:
                raise ValueError("default attribute must be a boolean")
        else:
            self.__default = None

    @property
    def encryption(self):
        """Method of verifying integrity of the media file.
        Accepted values are either ``"sri"`` or ``"pgp-signature"``

        When specified with :attr:`AlternateMedia.signature`, a ``<podcast:integrity>``
        tag will be appended to the parent ``<podcast:alternateEnclosure>``.

        Check `podcast-namespace Integrity tag
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#integrity>`_
        for more details.

        :type: :obj:`str`
        """
        return self.__encryption

    @encryption.setter
    def encryption(self, encryption):
        if encryption is not None:
            if encryption not in ["sri", "pgp-signature"]:
                raise ValueError(
                    "encryption value can either be 'sri' or 'pgp-signature'"
                )

            self.__encryption = encryption

        else:
            self.__encryption = None

    @property
    def signature(self):
        """Media signature based on the selected :attr:`AlternateMedia.encryption`:
        ``"sri"`` or ``"pgp-signature"``

        When specified with :attr:`AlternateMedia.encryption`, a ``<podcast:integrity>``
        tag will be appended to the parent ``<podcast:alternateEnclosure>``.

        Check `podcast-namespace Integrity tag
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#integrity>`_
        for more details.

        :type: :obj:`str`
        """

        return self.__signature

    @signature.setter
    def signature(self, signature):
        if signature:
            if not isinstance(signature, str):
                raise TypeError("signature can only be a string")

            self.__signature = signature

        else:
            self.__signature = None

    def add_source(self, uri, type=None):
        """Append a new source to the AlternateMedia object. Use this method
        instead of editing :attr:`AlternateMedia.sources <pod2gen.AlternateMedia.sources>`

        :param uri: The uri where the media file to append resides
        :type uri: str
        :param type: (Optional) MIME type of the media file to append
        :type type: str or None
        """

        if not isinstance(uri, str):
            raise TypeError("uri must be a string.")

        if uri in self.__sources:
            raise ValueError(
                "uri %s already defined. "
                "Please use edit_source method for modifications." % uri
            )

        if type is not None:
            if not isinstance(type, str):
                raise TypeError(
                    "type parameter of add_source method must be a string or None."
                )

            if not type:
                type = None

        if validators.url(uri) is not True:
            if not uri.startswith("ipfs://"):
                raise ValueError("Not a valid url: %s" % uri)

        self.__sources[uri] = type

    def delete_source(self, uri):
        """Delete a source from :attr:`AlternateMedia.sources`. Use this method
        instead of editing :attr:`AlternateMedia.sources <pod2gen.AlternateMedia.sources>`.

        :param uri: The media file uri to remove
        :type uri: str
        """

        if uri in self.__sources:
            del self.__sources[uri]

    def edit_source_content(self, uri, type):
        """Edit a media file content type from :attr:`AlternateMedia.sources`.
        Use this method instead of editing :attr:`AlternateMedia.sources <pod2gen.AlternateMedia.sources>`.

        :param uri: The media file uri to edit.
        :type uri: str
        :param type: The media file content type.
        :type type: str
        """

        if uri not in self.__sources:
            raise LookupError("uri %s does not exists within %r" % (uri, self))

        if type is not None and not isinstance(type, str):
            raise TypeError(
                "type parameter of edit_source_content method must be a string or None."
            )

        if not type:
            type = None

        self.__sources[uri] = type

    @property
    def sources(self):
        """A dictionary with uri as keys and content type as values. Each key/value pair
        represents a media file; the key is the uri where the media file resides and the
        value represents the content type. The content type is optional and :obj:`None` by default.
        It is useful if the transport mechanism is different than the type mentioned for
        the parent AlternateMedia object.

        For each source, a ``<podcast:source>`` is appended to the parent ``<podcast:alternateEnclosure>``.
        when generating the RSS feed.

        Check `podcast-namespace Source
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#source>`_
        for more details about the ``<podcast:source>`` tag.

        :type: :obj:`dict`
            { ``uri``: :obj:`str` : ``content_type``: :obj:`str` or :obj:`None` }
        """
        return self.__sources

    @sources.setter
    def sources(self, sources):
        old_sources = self.__sources

        if not isinstance(sources, dict):
            raise TypeError(
                "sources parameter must be a dictionary with uris "
                "as keys and contentType/None as values."
            )

        try:
            self.__sources = {}
            for k, v in sources.items():
                self.add_source(k, v)
        except Exception as e:
            self.__sources = old_sources
            raise e

    def __repr__(self):
        if self.title:
            return "AlternateMedia(title=%s) at %s" % (self.title, id(self))
        return "AlternateMedia(type=%s, length=%s) at %s" % (
            self.type,
            self.length,
            id(self),
        )



# -*- coding: utf-8 -*-
"""
    pod2gen.location
    ~~~~~~~~~~~~~~~

    This module contains Location, which represents a channel or an episode location.

    :copyright: 2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import re
import warnings

from lxml import etree


class Location(object):
    """Class representing a Location object.

    By using this class, you can be sure that the location object is formatted correctly.
    and will be correctly used by the associated :class:`~.pod2gen.Podcast` or
    :class:`~.pod2gen.Episode` object.

    Coordinates parsing is in accordance with
    `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_
    and basic regular expression are used to parse the :attr:`~.Location.geo` atrribute

    See `podcast-namespace Location
    <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#location>`_
    for an overview of the Location tag.

    Although `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_ accepts unknow fields
    like the ``"bar"`` attribute in the geo uri ``"geo:70,20;foo=1.00;bar=white"``,
    pod2gen will not accept it and will raise an error.

    Example::

        >>> from pod2gen import Location
        >>> l = Location("My location", osm="W5013364", geo="geo:37.786971,-122.399677,300;u=100")
        >>> l.text
        'My location'
        >>> l.osm
        'W5013364'
        >>> l.geo
        'geo:37.786971,-122.399677,300;u=100'
        >>> l.latitude
        37.786971
        >>> l.longitude
        -122.399677
        >>> l.altitude
        300
        >>> l.uncertainty
        100
        >>> # Updating geo URI by changing one coordinate
        >>> l.altitude = 400
        >>> l.geo
        'geo:37.786971,-122.399677,400;u=100'
        >>> # Creating a location with coordinate
        >>> l = Location("A second location", latitude=37.786971, longitude=-122.399677)
        >>> l.geo
        'geo:37.786971,-122.399677'
        >>> l.uncertainty = 50
        >>> l.geo
        'geo:37.786971,-122.399677;u=50'
    """

    _accepted_osm_regex = r"^[NRW]\d+(\#\d+)?$"
    _accepted_geo_regex = r"^geo:-?\d+(\.\d*)?,-?\d+(\.\d*)?(,-?\d+(\.\d*)?)?(;crs=[\w\d]+)?(;u=\d+(\.\d*)?)?$"
    _know_crs = ["wgs84"]

    _redundant_parameter_warning = (
        "latitude or longitude or altitude or crs or uncertainty "
        "parameters will be ignored when specifying a geo parameter"
    )

    def __init__(
        self,
        text,
        osm=None,
        geo=None,
        latitude=None,
        longitude=None,
        altitude=None,
        crs=None,
        uncertainty=None,
    ):
        """Create new Location object. See the class description of
        :class:´~pod2gen.transcript.Location`.

        :param text: The tag node value. This is the location to display.
        :type text: str
        :param osm: (Optional) The OpenStreetMap reference of the location.
        :type osm: str or None
        :param geo: (Optional) A geo URI, conformant to RFC 5870.
        :type geo: str or None
        :param latitude: (Optional) The location latitude.
        :type latitude: float or int
        :param longitude: (Optional) The location longitude.
        :type longitude: float or int
        :param altitude: (Optional) The location altitude.
        :type altitude: float or int
        :param crs: (Optional) The Coordinate Reference System Identification (wgs84).
        :type crs: str or None
        :param uncertainty: (Optional) the amount of uncertainty in the location (in meters).
        :type uncertainty: float or int
        """

        self._text = None
        self._osm = None
        self._geo = None
        self._latitude = None
        self._longitude = None
        self._altitude = None
        self._crs = None
        self._uncertainty = None

        self.text = text
        self.osm = osm

        if geo:
            if latitude or longitude or altitude or crs or uncertainty:
                warnings.warn(self._redundant_parameter_warning, stacklevel=2)
        else:
            if latitude and longitude:
                geo = self.build_geo_uri(
                    latitude, longitude, altitude, crs, uncertainty
                )
            elif bool(latitude) ^ bool(longitude):
                raise ValueError(
                    "Either specify a geo uri or both latitude and longitude"
                )

        self.geo = geo

    def build_geo_uri(
        self, latitude, longitude, altitude=None, crs=None, uncertainty=None
    ):
        """Create a geo URI from coordinates inputs.

        :param latitude: The location latitude.
        :type latitude: float or int
        :param longitude: The location longitude.
        :type longitude: float or int
        :param altitude: (Optional) The location altitude.
        :type altitude: float or int
        :param crs: (Optional) The Coordinate Reference System Identification
            (default is ``"wgs84"``).
        :type crs: str
        :param uncertainty: (Optional) the amount of uncertainty in the location (in meters).
        :type uncertainty: float or int

        :returns: :obj:`str`
        """

        if not latitude or not longitude:
            return None

        geo = "geo:%s,%s" % (latitude, longitude)
        if altitude:
            geo = "%s,%s" % (geo, altitude)
        if crs:
            geo = "%s;crs=%s" % (geo, crs)
        if uncertainty:
            geo = "%s;u=%s" % (geo, uncertainty)

        return geo

    def _update_from_geo_uri(self, reset_if_empty=True):
        geo = self.geo
        if geo:
            elements = re.split("[;,:]", geo)
            self.latitude = elements[1]
            self.longitude = elements[2]

            for i in range(3, len(elements)):
                item = elements[i]
                if item.startswith("crs="):
                    self.crs = item[4:]
                elif item.startswith("u="):
                    self.uncertainty = item[2:]
                else:
                    self.altitude = item
        elif reset_if_empty:
            self._latitude = None
            self._longitude = None
            self._altitude = None
            self._crs = None
            self._uncertainty = None

    def _update_from_parameters(
        self, latitude=None, longitude=None, altitude=None, crs=None, uncertainty=None
    ):
        if not latitude:
            latitude = self.latitude
        if not longitude:
            longitude = self.longitude
        if not altitude:
            altitude = self.altitude
        if not crs:
            crs = self.crs
        if not uncertainty:
            uncertainty = self.uncertainty

        geo = self.build_geo_uri(latitude, longitude, altitude, crs, uncertainty)
        self._geo = geo

    def rss_entry(self):
        """Create an RSS ``<podcast:location>`` tag using lxml's etree and return it.

        This is primarily used by both :class:`~.pod2gen.Podcast` and
        :class:`~.pod2gen.Episode` as the location tag parent is either
        the channel (podcast) or item (episode)

        :RSS: ``<podcast:location>``

        :returns: :class:`lxml.etree.Element`
        """

        PODCAST_NS = "https://podcastindex.org/namespace/1.0"

        entry = etree.Element("{%s}location" % PODCAST_NS)
        entry.text = etree.CDATA(self.text)

        if self.geo:
            entry.attrib["geo"] = self.geo
        if self.osm:
            entry.attrib["osm"] = self.osm

        return entry

    @property
    def text(self):
        """The tag ``<podcast:transcript>`` node value. It is meant for podcast
        apps to display the name of the location that the podcast or episode is about.
        It is for display purposes only.

        Check `podcast-namespace Location node value
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#node-value-4>`_
        for more details.

        :type: :obj:`str`
        """
        return self._text

    @text.setter
    def text(self, text):
        if not text:
            raise ValueError("text field of location objects cannot be None or empty")

        if len(text) > 128:
            raise ValueError("Location display text should not exceed 128 characters")

        self._text = text

    @property
    def osm(self):
        """The `Open Street Map <https://en.wikipedia.org/wiki/OpenStreetMap>`_
        identifier of the place.

        Check `podcast-namespace Location osm attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-6>`_
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self._osm

    @osm.setter
    def osm(self, osm):
        if osm is None:
            self._osm = None
        else:
            if not re.match(self._accepted_osm_regex, osm):
                raise ValueError("%s is not a valid osm reference" % osm)

            self._osm = osm

    @property
    def geo(self):
        """A geo URI, conformant to `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_.

        The geo URI indicates all of latitude, longitude, altitude and precision uncertainty of
        the place.

        Check `podcast-namespace Location geo attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-6>`_
        for more details.

        :type: :obj:`str`
        """
        return self._geo

    @geo.setter
    def geo(self, geo):
        if geo:
            if not re.match(self._accepted_geo_regex, geo):
                raise ValueError("%s is not a valid geo uri" % geo)

            self._geo = geo
            self._update_from_geo_uri()

        else:
            self._geo = None
            self._update_from_geo_uri(reset_if_empty=True)

    @property
    def latitude(self):
        """The latitude in the geo URI, conformant to
        `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_.

        :type: :obj:`float`
        """
        return self._latitude

    @latitude.setter
    def latitude(self, latitude):
        if latitude:
            if not isinstance(latitude, int) and not isinstance(latitude, float):
                try:
                    latitude = float(latitude)
                except:
                    raise ValueError(
                        "Latitude value %r could not be converted into a numeric value"
                        % latitude
                    )

            if latitude - int(latitude) == 0:
                latitude = int(latitude)

            self._latitude = latitude
        else:
            self._latitude = None

        self._update_from_parameters()

    @property
    def longitude(self):
        """The longitude in the geo URI, conformant to
        `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_.

        :type: :obj:`float`
        """
        return self._longitude

    @longitude.setter
    def longitude(self, longitude):
        if longitude:
            if not isinstance(longitude, int) and not isinstance(longitude, float):
                try:
                    longitude = float(longitude)
                except:
                    raise ValueError(
                        "Longitude value %r could not be converted into a numeric value"
                        % longitude
                    )

            if longitude - int(longitude) == 0:
                longitude = int(longitude)

            self._longitude = longitude
        else:
            self._longitude = None

        self._update_from_parameters()

    @property
    def altitude(self):
        """The altitude in the geo URI, conformant to
        `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_

        :type: :obj:`float` or :obj:`None`
        """
        return self._altitude

    @altitude.setter
    def altitude(self, altitude):
        if altitude:
            if not isinstance(altitude, int) and not isinstance(altitude, float):
                try:
                    altitude = float(altitude)
                except:
                    raise ValueError(
                        "Altitude value %r could not be converted into a numeric value"
                        % altitude
                    )

            if altitude - int(altitude) == 0:
                altitude = int(altitude)

            self._altitude = altitude
        else:
            self._altitude = None

        self._update_from_parameters()

    @property
    def crs(self):
        """The coordinate reference system in the geo URI, conformant to
        `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_
        The default coordinate reference system used is the
        `World Geodetic System 1984 (WGS-84) <https://en.wikipedia.org/wiki/World_Geodetic_System>`_.

        :type: :obj:`str` or :obj:`None`
        """
        return self._crs

    @crs.setter
    def crs(self, crs):
        if crs:
            crs = crs.lower()
            if crs not in self._know_crs:
                warnings.warn(
                    "The %s coordinate system is unknown or not conformant to RFC 5870 yet"
                    % crs,
                    stacklevel=2,
                )
            self._crs = crs
        else:
            self._crs = None

        self._update_from_parameters()

    @property
    def uncertainty(self):
        """The uncertainty in the geo URI, conformant to
        `RFC 5870 <https://datatracker.ietf.org/doc/html/rfc5870>`_

        :type: :obj:`float` or obj:`None`
        """
        return self._uncertainty

    @uncertainty.setter
    def uncertainty(self, uncertainty):
        if uncertainty:
            if not isinstance(uncertainty, int) and not isinstance(uncertainty, float):
                try:
                    uncertainty = float(uncertainty)
                except:
                    raise ValueError(
                        "Uncertainty value %r could not be converted into a numeric value"
                        % uncertainty
                    )

            if uncertainty < 0:
                raise ValueError(
                    "Uncertainty cannot be negative. Received %r." % uncertainty
                )

            if uncertainty - int(uncertainty) == 0:
                uncertainty = int(uncertainty)

            self._uncertainty = uncertainty
        else:
            self._uncertainty = None

        self._update_from_parameters()

    def __repr__(self):
        return "Location(text=%s, osm=%s, geo=%s)" % (self.text, self.osm, self.geo)



# -*- coding: utf-8 -*-
"""
    pod2gen.soundbite
    ~~~~~~~~~~~~~~~

    This module contains Soundbite, which represents an episode soundbite.

    :copyright: 2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""


class Soundbite(object):
    """Class representing an Episode soundbite object.

    By using this class, you can be sure that the soundbite object is formatted correctly
    and will be correctly used by the associated :class:`~.pod2gen.Episode` object.

    :class:`.Soundbite` objects can be attached to :class:`.Episode` objects
    using this method. Soundbites are stored in :attr:`.Episode.soundbites`.

    See `podcast-namespace Soundbite
    <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#soundbite>`_
    for an overview of the Soundbite tag.

    Example::

        >>> from pod2gen import Soundbite, Episode
        >>> s = Soundbite(1234.5, 42.25, text="Why the Podcast Namespace Matters")
        >>> s.start_time
        1234.5
        >>> s.duration
        42.25
        >>> s.text
        'Why the Podcast Namespace Matters'
        >>> e = Episode()
        >>> e.add_soundbite(s)
        >>> s = Soundbite(73.0, 60)
        >>> s.start_time
        73.0
        >>> s.duration
        60
        >>> e.add_soundbite(s)
        >>> len(e.soundbites)
        2
        >>> # Editing attributes
        >>> s.start_time = 80
        >>> s.start_time
        80
        >>> s.text = "Soundbite example sample"
        >>> s.text
        'Soundbite example sample'
    """

    def __init__(self, start_time, duration, text=None):
        """Create new Soundbite object. See the class description of
        :class:´~pod2gen.soundbite.Soundbite`.

        :param start_time: The time where the soundbite begins in seconds.
        :type start_time: float or int
        :param duration: How long is the soundbite (recommended between 15 and 120 seconds).
        :type duration: float or int
        :param text: (Optional) Free form string to specify a title for the soundbite.
        :type text: str
        """

        self.__start_time = None
        self.__duration = None
        self.__text = None

        self.start_time = start_time
        self.duration = duration
        self.text = text

    @property
    def start_time(self):
        """The time where the soundbite begins in seconds.

        Check https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-4
        for more details.

        :type: :obj:`float`
        """
        return self.__start_time

    @start_time.setter
    def start_time(self, start_time):
        if start_time:
            if not isinstance(start_time, int) and not isinstance(start_time, float):
                try:
                    start_time = float(start_time)
                except:
                    raise ValueError(
                        "start_time value %r could not be converted into a numeric value"
                        % start_time
                    )

            if start_time - int(start_time) == 0:
                start_time = int(start_time)

            self.__start_time = start_time
        else:
            raise ValueError("start_time value cannot be None")

    @property
    def duration(self):
        """How long is the soundbite in seconds.

        Check https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-4
        for more details.

        :type: :obj:`float`
        """
        return self.__duration

    @duration.setter
    def duration(self, duration):
        if duration is not None:
            if not isinstance(duration, int) and not isinstance(duration, float):
                try:
                    duration = float(duration)
                except:
                    raise ValueError(
                        "duration value %r could not be converted into a numeric value"
                        % duration
                    )

            if duration - int(duration) == 0:
                duration = int(duration)

            self.__duration = duration
        else:
            raise ValueError("duration value cannot be None")

    @property
    def text(self):
        """A free form string from the podcast creator to specify
        a title for the soundbite.

        Check https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#node-value-2
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__text

    @text.setter
    def text(self, text):
        if text is not None:
            if not isinstance(text, str):
                raise ValueError("Soundbite text must be a valid string")

            if len(text) > 128:
                raise ValueError("Soundbite text must not exceed 128 characters")

            self.__text = text
        else:
            self.__text = None

    def __repr__(self):
        if self.__text:
            return "Soundbite(start_time=%s, duration=%s, text=%s)" % (
                self.start_time,
                self.duration,
                self.text,
            )
        return "Soundbite(start_time=%s, duration=%s)" % (
            self.start_time,
            self.duration,
        )



# -*- coding: utf-8 -*-
"""
    pod2gen.episode
    ~~~~~~~~~~~~~~

    :copyright: 2013, Lars Kiesow <lkiesow@uos.de>; 2016, Thorben Dahl
        <thorben@sjostrom.no>; 2021, Slim Beji <mslimbeji@gmail.com>

    :license: FreeBSD and LGPL, see license.* for more details.
"""
import warnings
from datetime import datetime
from urllib.parse import urlparse

import dateutil.parser
import dateutil.tz
import validators
from lxml import etree

from pod2gen.constants import (
    EPISODE_TYPE_BONUS,
    EPISODE_TYPE_FULL,
    EPISODE_TYPE_TRAILER,
)
from pod2gen.util import formatRFC2822, listToHumanreadableStr
from pod2gen.warnings import NotSupportedByItunesWarning

from .alternate_media import AlternateMedia
from .license import License
from .location import Location
from .person import Person
from .soundbite import Soundbite
from .transcript import Transcript


class Episode(object):
    """Class representing an episode in a podcast. Corresponds to an RSS Item.

    When creating a new Episode, you can populate any attribute
    using keyword arguments. Use the attribute's name on the left side of
    the equals sign and its value on the right side. Here's an example::

        >>> # This...
        >>> ep = Episode()
        >>> ep.title = "Exploring the RTS genre"
        >>> ep.summary = "Tory and I talk about a genre of games we've " + \\
        ...              "never dared try out before now..."
        >>> # ...is equal to this:
        >>> ep = Episode(
        ...    title="Exploring the RTS genre",
        ...    summary="Tory and I talk about a genre of games we've "
        ...            "never dared try out before now..."
        ... )

    :raises: TypeError if you try to set an attribute which doesn't exist,
             ValueError if you set an attribute to an invalid value.

    You must have filled in either :attr:`.title` or :attr:`.summary` before
    the RSS can be generated.

    To add an episode to a podcast::

        >>> from pod2gen import Episode, Podcast
        >>> p = Podcast()
        >>> episode = Episode()
        >>> p.episodes.append(episode)

    You may also replace the last two lines with a shortcut::

        >>> episode = p.add_episode(Episode())


    .. seealso::

       :doc:`/usage_guide/episodes`
          A friendlier introduction to episodes.
    """

    def __init__(self, **kwargs):
        # RSS
        self.__authors = []
        self.summary = None
        """The summary of this episode, in a format that can be parsed by
        XHTML parsers.

        If your summary isn't fit to be parsed as XHTML, you can use
        :func:`pod2gen.htmlencode() <pod2gen.util.htmlencode>` 
        to fix the text, like this::

            >>> from pod2gen import htmlencode
            >>> ep.summary = htmlencode("We spread lots of love <3")
            >>> ep.summary
            'We spread lots of love &lt;3'

        In iTunes, the summary is shown in a separate window that appears when
        the "circled i" in the Description column is clicked. This field can be
        up to 4000 characters in length.

        See also :py:attr:`.Episode.subtitle` and
        :py:attr:`.Episode.long_summary`.

        :type: :obj:`str` which can be parsed as XHTML.
        :RSS: ``<description>``"""

        self.long_summary = None
        """A long (read: full) summary, which supplements the shorter
        :attr:`~pod2gen.Episode.summary`. Like summary, this must be compatible
        with XHTML parsers; use :func:`pod2gen.htmlencode() <pod2gen.util.htmlencode>` 
        if this isn't HTML.

        This attribute should be seen as a full, longer variation of
        summary if summary exists. Even then, the long_summary should be
        independent from summary, in that you only need to read one of them.
        This means you may have to repeat the first sentences.

        :type: :obj:`str` which can be parsed as XHTML.
        :RSS: ``<content:encoded>`` or ``<description>``
        """

        self.__media = None

        self.__alternate_media_list = []

        self.__license = None
        """License object representing the license applied to the audio/video content of 
        the episode.
        """

        self.id = None
        """This episode's globally unique identifier.

        If not present, the URL of the enclosed media is used. This is usually
        the best way to go, **as long as the media URL doesn't change**.

        Set the id to boolean ``False`` if you don't want to associate any id to
        this episode.

        It is important that an episode keeps the same ID until the end of time,
        since the ID is used by clients to identify which episodes have been
        listened to, which episodes are new, and so on. Changing the ID causes
        the same consequences as deleting the existing episode and adding a
        new, identical episode.

        Note that this is a GLOBALLY unique identifier. Thus, not only must it
        be unique in this podcast, it must not be the same ID as any other
        episode for any podcast out there. To ensure this, you should use a
        domain which you own (for example, use something like
        http://example.org/podcast/episode1 if you own example.org).

        :type: :obj:`str`, :obj:`None` to use default or :obj:`False` to leave
            out.
        :RSS: ``<guid>``
        """

        self.link = None
        """The link to the full version of this episode's :attr:`.summary`.
        Remember to start the link with the scheme, e.g. https://.

        :type: :obj:`str`
        :RSS: ``<link>``
        """

        self.__publication_date = None

        self.title = None
        """This episode's human-readable title.
        Title is mandatory and should not be blank.

        :type: :obj:`str`
        :RSS: ``<title>``
        """

        # ITunes tags
        # http://www.apple.com/itunes/podcasts/specs.html#rss
        self.__withhold_from_itunes = False

        self.__image = None

        self.__itunes_duration = None

        self.__explicit = None

        self.is_closed_captioned = False
        """Whether this podcast includes a video episode with embedded `closed
        captioning`_ support. Defaults to ``False``.

        :type: :obj:`bool`
        :RSS: ``<itunes:isClosedCaptioned>``

        .. _closed captioning: https://en.wikipedia.org/wiki/Closed_captioning
        """

        self.__position = None

        self.__episode_number = None

        self.__episode_name = None

        self.__season = None

        self.__season_name = None

        self.__chapters_json = None
        """Link to an external file containing chapter data for the episode.
        
        See the below link for a real example:
        https://github.com/Podcastindex-org/podcast-namespace/blob/main/chapters/example.json
        
        See the below link for a detailed explanation about the Chapters tag:
        https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#chapters
        """

        self.__soundbites = []
        """List of soundbites for a podcast episode. The intended use includes episodes previews,
        discoverability, audiogram generation, episode highlights, etc. It should be assumed that
        the audio/video source of the soundbite is the audio/video given in the item's <enclosure>
        element.
        
        See the below link for a detailed explanation about the Soundbite tag:
        https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#soundbite
        """

        self.__persons = []
        """List of persons of interest to the episode. It is primarily intended to identify 
        people like hosts, co-hosts and guests. Although, it is flexible enough to allow fuller
        credits to be given using the roles and groups that are listed in the Podcast Taxonomy
        Project.
        
        See the below link for a detailed explanation about the Person tag:
        https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#person
        """

        self.__episode_type = EPISODE_TYPE_FULL

        self.subtitle = None
        """A short subtitle.

        This is shown in the Description column in iTunes.
        The subtitle displays best if it is only a few words long.

        :type: :obj:`str`
        :RSS: ``<itunes:subtitle>``
        """

        self.__transcripts = []
        """List of episode transcript links.
        
        The transcript tag exists in the podcast namespace
        https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#transcript
        https://podcastindex.org/namespace/1.0
        """

        self.__location = None
        """The location of the episode.

        See https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#location
        for an overview of the location tag

       :type: :obj:`pod2gen.Location`
        """

        # It is time to assign the keyword arguments
        for attribute, value in kwargs.items():
            if hasattr(self, attribute):
                setattr(self, attribute, value)
            else:
                raise TypeError(
                    "Keyword argument %s (with value %s) not "
                    "recognized!" % (attribute, value)
                )

    def rss_entry(self):
        """Create an RSS item using lxml's etree and return it.

        This is primarily used by :class:`~pod2gen.Podcast` when generating the
        podcast's RSS feed.

        :returns: :class:`lxml.etree.Element`
        :RSS: ``<item>``
        """

        ITUNES_NS = "http://www.itunes.com/dtds/podcast-1.0.dtd"
        DUBLIN_NS = "http://purl.org/dc/elements/1.1/"
        PODCAST_NS = "https://podcastindex.org/namespace/1.0"

        entry = etree.Element("item")

        if not (self.title or self.summary):
            raise ValueError(
                "Required fields not set, make sure either " "title or summary is set!"
            )

        if self.title:
            title = etree.SubElement(entry, "title")
            title.text = self.title

        if self.link:
            link = etree.SubElement(entry, "link")
            link.text = self.link

        if self.summary or self.long_summary:
            if self.summary and self.long_summary:
                # Both are present, so use both content and description
                description = etree.SubElement(entry, "description")
                description.text = etree.CDATA(self.summary)
                content = etree.SubElement(
                    entry, "{%s}encoded" % "http://purl.org/rss/1.0/modules/content/"
                )
                content.text = etree.CDATA(self.long_summary)
            else:
                # Only one is present, use description because of support
                description = etree.SubElement(entry, "description")
                description.text = etree.CDATA(self.summary or self.long_summary)

        if self.__authors:
            authors_with_name = [a.name for a in self.__authors if a.name]
            if authors_with_name:
                # We have something to display as itunes:author, combine all
                # names
                itunes_author = etree.SubElement(entry, "{%s}author" % ITUNES_NS)
                itunes_author.text = listToHumanreadableStr(authors_with_name)
            if len(self.__authors) > 1 or not self.__authors[0].email:
                # Use dc:creator, since it supports multiple authors (and
                # author without email)
                for a in self.__authors or []:
                    author = etree.SubElement(entry, "{%s}creator" % DUBLIN_NS)
                    if a.name and a.email:
                        author.text = "%s <%s>" % (a.name, a.email)
                    elif a.name:
                        author.text = a.name
                    else:
                        author.text = a.email
            else:
                # Only one author and with email, so use rss author
                author = etree.SubElement(entry, "author")
                author.text = str(self.__authors[0])

        if self.id:
            rss_guid = self.id
        elif self.__media and self.id is None:
            rss_guid = self.__media.url
        else:
            # self.__rss_guid was set to boolean False, or no enclosure
            rss_guid = None
        if rss_guid:
            guid = etree.SubElement(entry, "guid")
            guid.text = rss_guid
            guid.attrib["isPermaLink"] = "false"

        if self.__media:
            enclosure = etree.SubElement(entry, "enclosure")
            enclosure.attrib["url"] = self.__media.url
            enclosure.attrib["length"] = str(self.__media.size)
            enclosure.attrib["type"] = self.__media.type

            if self.__media.duration:
                duration = etree.SubElement(entry, "{%s}duration" % ITUNES_NS)
                duration.text = self.__media.duration_str

        if self.__alternate_media_list:
            for alternate_media in self.__alternate_media_list:
                alternate_enclosure = alternate_media.rss_entry()
                entry.append(alternate_enclosure)

        if self.__license:
            license_element = etree.SubElement(entry, "{%s}license" % PODCAST_NS)
            license_element.text = self.__license.identifier
            if self.__license.url:
                license_element.attrib["url"] = self.__license.url

        if self.__publication_date:
            pubDate = etree.SubElement(entry, "pubDate")
            pubDate.text = formatRFC2822(self.__publication_date)

        if self.__withhold_from_itunes:
            # It is True, so include element - otherwise, don't include it
            block = etree.SubElement(entry, "{%s}block" % ITUNES_NS)
            block.text = "Yes"

        if self.__image:
            image = etree.SubElement(entry, "{%s}image" % ITUNES_NS)
            image.attrib["href"] = self.__image

        if self.__explicit is not None:
            explicit = etree.SubElement(entry, "{%s}explicit" % ITUNES_NS)
            explicit.text = "yes" if self.__explicit else "no"

        if self.is_closed_captioned:
            is_closed_captioned = etree.SubElement(
                entry, "{%s}isClosedCaptioned" % ITUNES_NS
            )
            is_closed_captioned.text = "Yes"

        if self.__episode_type != EPISODE_TYPE_FULL:
            episode_type = etree.SubElement(entry, "{%s}episodeType" % ITUNES_NS)
            episode_type.text = self.__episode_type

        if self.__season is not None:
            season = etree.SubElement(entry, "{%s}season" % ITUNES_NS)
            season.text = str(self.__season)

            season = etree.SubElement(entry, "{%s}season" % PODCAST_NS)
            season.text = str(self.__season)
            if self.__season_name:
                season.attrib["name"] = self.__season_name

        if self.__chapters_json is not None:
            chapters = etree.SubElement(entry, "{%s}chapters" % PODCAST_NS)
            chapters.attrib["url"] = self.__chapters_json
            chapters.attrib["type"] = "application/json+chapters"

        if self.soundbites:
            for soundbite in self.soundbites:
                soundbite_element = etree.SubElement(
                    entry, "{%s}soundbite" % PODCAST_NS
                )
                soundbite_element.attrib["startTime"] = str(soundbite.start_time)
                soundbite_element.attrib["duration"] = str(soundbite.duration)
                if soundbite.text:
                    soundbite_element.text = soundbite.text

        if self.persons:
            for person in self.persons:
                person_element = person.rss_entry()
                entry.append(person_element)

        if self.__position is not None and self.__position >= 0:
            order = etree.SubElement(entry, "{%s}order" % ITUNES_NS)
            order.text = str(self.__position)

        if self.__episode_number is not None:
            episode_number = etree.SubElement(entry, "{%s}episode" % ITUNES_NS)
            # Convert via int, since we stored the original as-is
            episode_number.text = str(int(self.__episode_number))

            episode_number = etree.SubElement(entry, "{%s}episode" % PODCAST_NS)
            # Convert via int, since we stored the original as-is
            episode_number.text = str(int(self.__episode_number))
            if self.__episode_name:
                episode_number.attrib["display"] = self.__episode_name

        if self.subtitle:
            subtitle = etree.SubElement(entry, "{%s}subtitle" % ITUNES_NS)
            subtitle.text = self.subtitle

        if self.transcripts:
            for transcript in self.transcripts:
                transcriptElem = etree.SubElement(entry, "{%s}transcript" % PODCAST_NS)
                transcriptElem.attrib["url"] = transcript.url
                transcriptElem.attrib["type"] = transcript.type
                if transcript.language:
                    transcriptElem.attrib["language"] = transcript.language
                if transcript.is_caption:
                    transcriptElem.attrib["rel"] = "captions"

        if self.location:
            locationElem = self.location.rss_entry()
            entry.append(locationElem)

        return entry

    @property
    def authors(self):
        """List of :class:`~pod2gen.Person` that contributed to this
        episode.

        The authors don't need to have both name and email set. They're usually
        not displayed anywhere.

        .. note::

            You do not need to provide any authors for an episode if
            they're identical to the podcast's authors.

        Any value you assign to authors will be automatically converted to a
        list, but only if it's iterable (like tuple, set and so on). It is an
        error to assign a single :class:`~pod2gen.Person` object to this
        attribute::

            >>> # This results in an error
            >>> ep.authors = Person("John Doe", "johndoe@example.org")
            TypeError: Only iterable types can be assigned to authors, ...
            >>> # This is the correct way:
            >>> ep.authors = [Person("John Doe", "johndoe@example.org")]

        The initial value is an empty list, so you can use the list methods
        right away.

        Example::

            >>> # This attribute is just a list - you can for example append:
            >>> ep.authors.append(Person("John Doe", "johndoe@example.org"))
            >>> # Or assign a new list (discarding earlier authors)
            >>> ep.authors = [Person("John Doe", "johndoe@example.org"),
            ...               Person("Mary Sue", "marysue@example.org")]

        :type: :obj:`list` of :class:`~pod2gen.Person`
        :RSS: ``<author>`` or ``<dc:creator>``, and ``<itunes:author>`` elements
        """
        return self.__authors

    @authors.setter
    def authors(self, authors):
        try:
            self.__authors = list(authors)
        except TypeError:
            raise TypeError(
                "Only iterable types can be assigned to authors, "
                "%s given. You must put your object in a list, "
                "even if there's only one author." % authors
            )

    @property
    def publication_date(self):
        """The time and date this episode was first published.

        The value can be a :obj:`str`, which will be parsed and
        made into a :class:`datetime.datetime` object when assigned. You may
        also assign a :class:`datetime.datetime` object directly. In both cases,
        you must ensure that the value includes timezone information.

            >>> from pod2gen import Episode
            >>> from datetime import datetime
            >>> from dateutil.tz import UTC
            >>> my_episode = Episode()
            >>> my_episode.publication_date = datetime(1990, 9, 28, tzinfo=UTC)
            >>> my_episode.publication_date
            datetime.datetime(1990, 9, 28, 7, 0, tzinfo=tzutc())
            >>> my_episode.publication_date = "2020-09-28 18:30:00+00:00"
            >>> my_episode.publication_date
            datetime.datetime(2020, 9, 28, 18, 30, tzinfo=tzutc())

        :type: :obj:`str` (will be converted to and stored as
            :class:`datetime.datetime`) or :class:`datetime.datetime`.
        :RSS: ``<pubDate>``

        .. note::

           Don't use the media file's modification date as the publication
           date, unless they're the same. It looks very odd when an episode
           suddenly pops up in the feed, but it claims to be several hours old!
        """
        return self.__publication_date

    @publication_date.setter
    def publication_date(self, publication_date):
        if publication_date is not None:
            if isinstance(publication_date, str):
                publication_date = dateutil.parser.parse(publication_date)
            if not isinstance(publication_date, datetime):
                raise ValueError("Invalid datetime format")
            if publication_date.tzinfo is None:
                raise ValueError("Datetime object has no timezone info")
            self.__publication_date = publication_date
        else:
            self.__publication_date = None

    @property
    def media(self):
        """Get or set the :class:`~pod2gen.Media` object that is attached
        to this episode.

        Note that if :py:attr:`.id` is not set, the media's URL is used as
        the id. If you rely on this, you should make sure the URL never changes,
        since changing the id messes up with clients (they will think this
        episode is new again, even if the user has listened to it already).
        Therefore, you should only rely on this behaviour if you own the domain
        which the episodes reside on. If you don't, then you must set
        :py:attr:`.id` to an appropriate value manually.

        .. seealso::
           :ref:`pod2gen.Media-guide`
               for a more gentle introduction.

        :type: :class:`~pod2gen.Media`
        :RSS: ``<enclosure>`` and ``<itunes:duration>``
        """
        return self.__media

    @media.setter
    def media(self, media):
        if media is not None:
            # Test that the media quacks like a duck
            if (
                hasattr(media, "url")
                and hasattr(media, "size")
                and hasattr(media, "type")
            ):
                # It's a duck
                self.__media = media
            else:
                raise TypeError(
                    "The parameter media must have the attributes "
                    "url, size and type."
                )
        else:
            self.__media = None

    @property
    def alternate_media_list(self):
        """List of :class:`~pod2gen.AlternateMedia` objects that are attached
        to this episode.

        These elements are meant to provide different versions of, or companion media to the
        main ``<enclosure>`` file. This could be an audio only version of a video podcast to
        allow apps to switch back and forth between audio/video, lower (or higher) bitrate
        versions for bandwidth constrained areas, alternative codecs for different device
        platforms, alternate URI schemes and download types such as IPFS or WebTorrent,
        commentary tracks or supporting source clips, etc.

            >>> from pod2gen import AlternateMedia
            >>> my_episode.alternate_media_list

        :type: :obj:`list` of :class:`~pod2gen.AlternateMedia`
        :RSS: ``<podcast:alternateEnclosure>`` elements
        """
        return self.__alternate_media_list

    @alternate_media_list.setter
    def alternate_media_list(self, alternate_media_list):
        # Ensure it is a list of transcripts or empty list
        if alternate_media_list is None:
            alternate_media_list = []

        try:
            alternate_media_list = list(alternate_media_list)
        except:
            raise TypeError(
                "Expected an iterable of AlternateMedia, got %r" % alternate_media_list
            )

        for alternate_media in alternate_media_list:
            if not isinstance(alternate_media, AlternateMedia):
                raise TypeError(
                    "An AlternateMedia object must be used, got " "%r" % alternate_media
                )

        self.__alternate_media_list = alternate_media_list

    def add_alternate_media(self, new_media):
        """Shorthand method which adds a new :class:`~pod2gen.AlternateMedia` object to
        :attr:`~pod2gen.Episode.alternate_media_list`.

        An :class:`~pod2gen.AlternateMedia` object must be passed as a parameter to make
        sure all the required parameters are specified and valid. This is the easiest way
        to add a new media alternative to the episode.

        :param new_media: :class:`~pod2gen.AlternateMedia` object to add.
        :returns: AlternateMedia object passed to this function.

        Example::

            >>> from pod2gen import AlternateMedia
            >>> am = AlternateMedia(
            ...     "audio/mp4",
            ...     43200000,
            ...     bitrate=128000,
            ...     height=1080,
            ...     lang="en-US",
            ...     title="Standard",
            ...     rel="Off stage",
            ...     codecs="mp4a.40.2",
            ...     default=False,
            ...     encryption="sri",
            ...     signature="sha384-ExVqijgYHm15PqQqdXfW95x+Rs6C+d6E/ICxyQOeFevnxNLR/wtJNrNYTjIysUBo",
            ... )
            >>> my_episode.add_alternate_media(am)

        :param new_media: The AlternateMedia object to add
        :type new_media: :class:`.AlternateMedia`
        """
        if new_media is None:
            return

        if not isinstance(new_media, AlternateMedia):
            raise TypeError(
                "An AlternateMedia object must be used, got " "%r" % new_media
            )

        self.alternate_media_list.append(new_media)
        return new_media

    @property
    def license(self):
        """Get or set the :class:`~pod2gen.License` object that is attached
        to this episode.

            >>> from pod2gen import License
            >>> my_episode.license = License(
            ... "my-podcast-license-v1", "https://example.org/mypodcastlicense/full.pdf"
            ... )

        :type: :class:`~pod2gen.License`
        :RSS: ``<podcast:license>``
        """
        return self.__license

    @license.setter
    def license(self, license):
        if license is not None:
            if not isinstance(license, License):
                raise TypeError("license attribute must be a License object")

            self.__license = license

        else:
            self.__license = None

    @property
    def withhold_from_itunes(self):
        """Prevent this episode from appearing in the iTunes podcast directory.
        Note that the episode can still be found by inspecting the XML, so it is
        still public.

        One use case would be if you knew that this episode would get you kicked
        out from iTunes, should it make it there. In such cases, you can set
        withhold_from_itunes to ``True`` so this episode isn't published on
        iTunes, allowing you to publish it to everyone else while keeping your
        podcast on iTunes.

        This attribute defaults to ``False``, of course.

        :type: :obj:`bool`
        :RSS: ``<itunes:block>``
        """
        return self.__withhold_from_itunes

    @withhold_from_itunes.setter
    def withhold_from_itunes(self, withhold_from_itunes):
        if withhold_from_itunes is not None:
            if withhold_from_itunes is True or withhold_from_itunes is False:
                self.__withhold_from_itunes = withhold_from_itunes
            else:
                raise TypeError(
                    "withhold_from_itunes expects bool or None, "
                    "got %s" % withhold_from_itunes
                )
        else:
            self.__withhold_from_itunes = None

    @property
    def image(self):
        """The podcast episode's image, overriding the podcast's
        :attr:`~.Podcast.image`.

        This attribute specifies the absolute URL to the artwork for your
        podcast. iTunes prefers square images that are at least ``1400x1400``
        pixels.

        iTunes supports images in JPEG and PNG formats with an RGB color space
        (CMYK is not supported). The URL must end in ".jpg" or ".png"; a
        :class:`.NotSupportedByItunesWarning` will be issued if it doesn't.

        :type: :obj:`str`
        :RSS: ``<itunes:image>``

        .. note::

           If you change an episode’s image, you should also change the file’s
           name; iTunes doesn't check the actual file to see if it's changed.

           Additionally, the server hosting your cover art image must allow HTTP
           HEAD requests.

        .. warning::

            Almost no podcatchers support this. iTunes supports it only if you
            embed the cover in the media file (the same way you would embed
            an album cover), and recommends that you use Garageband's Enhanced
            Podcast feature.

            The podcast's image is used if this isn't supported.
        """
        return self.__image

    @image.setter
    def image(self, image):
        if image is not None:
            lowercase_image = str(image).lower()
            if not (lowercase_image.endswith((".jpg", ".jpeg", ".png"))):
                warnings.warn(
                    "Image filename must end with png or jpg, not "
                    "%s" % image.split(".")[-1],
                    NotSupportedByItunesWarning,
                    stacklevel=2,
                )
            self.__image = image
        else:
            self.__image = None

    @property
    def explicit(self):
        """Whether this podcast episode contains material which may be
        inappropriate for children.

        The value of the podcast's explicit attribute is used by default, if
        this is kept as ``None``.

        If you set this to ``True``, an "explicit" parental advisory
        graphic will appear in the Name column in iTunes. If the value is
        ``False``, the parental advisory type is considered Clean, meaning that
        no explicit language or adult content is included anywhere in this
        episode, and a "clean" graphic will appear.

            >>> my_episodde = Episode()
            >>> my_episodde.explicit is None
            True
            >>> my_episodde.explicit = True
            >>> my_episodde.explicit
            True

        :type: :obj:`bool`
        :RSS: ``<itunes:explicit>``
        """
        return self.__explicit

    @explicit.setter
    def explicit(self, explicit):
        if explicit is not None:
            # Force explicit to be bool, so no one uses "no" and expects False
            if explicit not in (True, False):
                raise ValueError('Invalid value "%s" for explicit tag' % explicit)
            self.__explicit = explicit
        else:
            self.__explicit = None

    @property
    def episode_type(self):
        """Indicate whether this is a full episode, or a bonus or trailer for
        an episode or a season.

        By default, the episode is taken to be a full episode.

        Use the constants ``EPISODE_TYPE_FULL``, ``EPISODE_TYPE_TRAILER`` or
        ``EPISODE_TYPE_BONUS`` (available to import from ``pod2gen``).

            >>> from pod2gen import EPISODE_TYPE_FULL, EPISODE_TYPE_TRAILER, EPISODE_TYPE_BONUS
            >>> full_episode.episode_type = EPISODE_TYPE_FULL
            >>> trailer_episode.episode_type = EPISODE_TYPE_TRAILER
            >>> bonus_episode.episode_type = EPISODE_TYPE_BONUS

        :type: One of the three constants mentioned.
        :RSS: ``<itunes:episodeType>``
        """
        return self.__episode_type

    @episode_type.setter
    def episode_type(self, episode_type):
        as_str = str(episode_type)
        if as_str not in (
            EPISODE_TYPE_FULL,
            EPISODE_TYPE_BONUS,
            EPISODE_TYPE_TRAILER,
        ):
            raise ValueError('Invalid episode_type value "%s"' % as_str)

        self.__episode_type = as_str

    @property
    def season(self):
        """The number of the season this episode belongs to.

        By default, the episode belongs to no season, indicated by this
        attribute being set to :obj:`None`.

        Some podcast applications may choose to show season numbers only when
        there is more than one season.

        This attribute is used by both the itunes and podcast namespaces.
        Affecting a value to it will generate two tags: ``<itunes:season>``
        and ``<podcast:season>``

        :type: :obj:`None` or positive :obj:`int`
        :RSS: ``<itunes:season>`` and ``<podcast:season>``
        """
        return self.__season

    @season.setter
    def season(self, season):
        if season is None:
            self.__season = None
        else:
            as_int = int(season)
            if as_int <= 0:
                raise ValueError(
                    "Season number must be a positive, non-zero "
                    'integer; not "%s"' % as_int
                )
            self.__season = as_int

    @property
    def season_name(self):
        """How should the season be displayed.

        This is the "name" of the season. If this attribute is present, applications
        are free to not show the season number to the end user, and may use it simply
        for chronological sorting and grouping purposes.

        This attribute is primarily used by podcast namespace.
        See `podcast-namespace Season
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-7>`_
        for more info.

        :type: :obj:`str`
        :RSS: ``<podcast:season>``

        """
        return self.__season_name

    @season_name.setter
    def season_name(self, label):
        if label is not None:
            if isinstance(label, str):
                if len(label) > 128:
                    raise ValueError("season_name must not exceed 128 characters")
                else:
                    self.__season_name = label
            else:
                raise ValueError("season_name must be a string; not " '"%s"' % label)
        else:
            self.__season_name = None

    @property
    def chapters_json(self):
        """Link to an external file containing chapter data for the episode.

            >>> my_episode.chapters_json = "https://example.com/episode1/chapters.json"

        Only absolute URLs are allowed, so make sure it starts with http:// or
        https://. The server should support HEAD-requests and byte-range
        requests.

        Ensure you quote parts of the URL that are not supposed to carry any
        special meaning to the browser, typically the name of your file.
        Common offenders include the slash character when not used to separate
        folders, the hash mark (#) and the question mark (?).

        :type: :obj:`str`
        :RSS: ``<podcast:chapters>``
        """
        return self.__chapters_json

    @chapters_json.setter
    def chapters_json(self, chapters_json):
        if chapters_json is not None:
            if validators.url(chapters_json) is not True:
                raise ValueError("Not a valid url: %s" % chapters_json)

            parsed_url = urlparse(chapters_json)
            if parsed_url.scheme not in ("http", "https"):
                warnings.warn(
                    "URL scheme %s is not supported by iTunes. Make sure "
                    "you use absolute URLs and HTTP or HTTPS." % parsed_url.scheme,
                    NotSupportedByItunesWarning,
                    stacklevel=2,
                )
            self.__chapters_json = chapters_json
        else:
            self.__chapters_json = None

    @property
    def position(self):
        """A custom position for this episode on the iTunes store page.

        If you would like this episode to appear first, set it to ``1``.
        If you want it second, set it to ``2``, and so on. If multiple episodes
        share the same position, they will be sorted by their
        :attr:`publication_date <.Episode.publication_date>`.

        To remove the order from the episode, set the position back to
        :obj:`None`.

        :type: :obj:`int`
        :RSS: ``<itunes:order>``
        """
        return self.__position

    @position.setter
    def position(self, position):
        if position is not None:
            self.__position = int(position)
        else:
            self.__position = None

    @property
    def episode_number(self):
        """This episode's number (within the season).

        This number is used to sort the episodes for
        :attr:`serial podcasts <.Podcast.is_serial>`. It can also be displayed
        to the user as the episode number. For
        :attr:`full episodes <episode_type>`, the episode numbers
        should be unique within each season.

        This is mandatory for full episodes of serial podcasts.

        This attribute is used by both the itunes and podcast namespaces.
        Affecting a value to it will generate two tags: ``<itunes:episode>``
        and ``<podcast:episode>``

        .. seealso::
           :ref:`pod2gen.Episode-organization`
              A friendlier introduction to episodes organization.

        :type: :obj:`None` or positive :obj:`int`
        :RSS: ``<itunes:episode>`` and ``<podcast:episode>``

        """
        return self.__episode_number

    @episode_number.setter
    def episode_number(self, episode_number):
        if episode_number is not None:
            as_integer = int(episode_number)
            if 0 < as_integer:
                # Store original (not int), to avoid confusion when setting
                self.__episode_number = episode_number
            else:
                raise ValueError(
                    "episode_number must be a positive, non-zero integer; not "
                    '"%s"' % episode_number
                )
        else:
            self.__episode_number = None

    @property
    def episode_name(self):
        """How should the episode be displayed.

        If this attribute is defined, podcast apps and aggregators are encouraged
        to show its value instead of the purely numerical episode_number.
        This attribute is expected to be a string.

        This attribute is primarily used by podcast namespace.
        See `podcast-namespace Episode
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-8>`_
        for more info.

        :type: :obj:`str`
        :RSS: ``<podcast:episode>``

        """
        return self.__episode_name

    @episode_name.setter
    def episode_name(self, label):
        if label is not None:
            if isinstance(label, str):
                if len(label) > 128:
                    raise ValueError("episode_name must not exceed 128 characters")
                else:
                    self.__episode_name = label
            else:
                raise ValueError("episode_name must be a string; not " '"%s"' % label)
        else:
            self.__episode_name = None

    @property
    def transcripts(self):
        """List of :class:`~pod2gen.Transcript` objects that
        contains the episode transcriptions data.

        .. seealso::
            the method :meth:`Episode.add_transcript() <.add_transcript>` to add
            :class:`~.pod2gen.Transcript` objects

        :type: :obj:`list` of :class:`~pod2gen.Transcript`
        :RSS: ``<podcast:transcript>`` elements

        """
        return self.__transcripts

    @transcripts.setter
    def transcripts(self, transcripts):
        # Ensure it is a list of transcripts or empty list
        if transcripts is None:
            transcripts = []

        try:
            transcripts = list(transcripts)
        except:
            raise TypeError("Expected an iterable of transcripts, got %r" % transcripts)

        for transcript in transcripts:
            if not isinstance(transcript, Transcript):
                raise TypeError(
                    "A Transcript object must be used, got " "%r" % transcript
                )

        self.__transcripts = transcripts

    def add_transcript(self, new_transcript):
        """Shorthand method which adds a new transcript to the episode.
        The transcript object must be passed as a parameter to make sure
        all the required parameters are specified and valid.
        This is the easiest way to add a transcript to an episode.

        :param new_transcript: :class:`.Transcript` object to add.
        :returns: Transcript object passed to this function.

        Example::

            >>> import pod2gen
            >>> transcript = Transcript("https://examples.com/transcript_sample.txt", "text/html")
            >>> # adding the transcript to the episode object my_episode
            >>> my_episode.add_transcript(transcript)

        :param new_transcript: The transcript object to add
        :type new_transcript: :class:`.Transcript`
        """
        self.transcripts.append(new_transcript)
        return new_transcript

    @property
    def location(self):
        """Object of :class:`~pod2gen.Location` with data about the episode location

        The location helps into parsing coordinaes and geo uri::

            >>> e.location = Location("Austin, TX", osm="R113314")
            >>> e.location.text
            'Austin, TX'
            >>> e.location.osm
            'R113314'
            >>> e.location.latitude = 30.2672
            >>> e.location.longitude = 97.7431
            >>> e.location.geo
            'geo:30.2672,97.7431'

        :type: :obj:`~pod2gen.Location`
        :RSS: ``<podcast:location>``
        """
        return self.__location

    @location.setter
    def location(self, location):
        if location is None:
            self.__location = None
        else:
            if not isinstance(location, Location):
                raise ValueError(
                    "Location attribute must be a pod2gen.Location object."
                    "Received %r instead" % location
                )
            else:
                self.__location = location

    def add_soundbite(self, soundbite):
        """Method that adds a new :class:`~pod2gen.Soundbite` object to the
        :class:`~pod2gen.Episode` object.

            >>> from pod2gen import Soundbite
            >>> s = Soundbite(1234.5, 42.25, text="Why the Podcast Namespace Matters")
            >>> my_episode.add_soundbite(s)

        :param soundbite: The soundbite object to add
        :type soundbite: :class:`~pod2gen.Soundbite`
        """

        if isinstance(soundbite, Soundbite):
            self.__soundbites.append(soundbite)
        else:
            raise ValueError("%r is not a soundbite object" % soundbite)

    @property
    def soundbites(self):
        """A list of Soundbite objects representing the episode soundbites

        .. seealso::
            the method :meth:`Episode.add_soundbite() <.add_soundbite>` to add
            :class:`~.pod2gen.Soundbite` objects

        :type: :obj:`list` of :class:`~pod2gen.Soundbite`
        :RSS: ``<podcast:soundbite>`` elements
        """
        return self.__soundbites

    @soundbites.setter
    def soundbites(self, soundbites):
        if soundbites is not None:
            try:
                soundbites = list(soundbites)
            except TypeError:
                raise ValueError("soundbites must be a list/tuple of Soundbite objects")

            for soundbite in soundbites:
                if not isinstance(soundbite, Soundbite):
                    raise ValueError(
                        "soundbites must be a list/tuple of soundbite objects"
                    )

            self.__soundbites = soundbites
        else:
            self.__soundbites = []

    def add_person(self, person):
        """Method that adds a new :class:`~pod2gen.Person` object to the
        :class:`~pod2gen.Episode` object.

            >>> from pod2gen import Person
            >>> myself = Person(
            ...     email="mslimbeji@gmail.com",
            ...     group="writing",
            ...     role="guest writer",
            ...     href="https://www.wikipedia/slimbeji",
            ...     img="http://example.com/images/slimbeji.jpg",
            ... )
            >>> my_episode.add_person(myself)

        :param person: The person object to add
        :type person: :class:`~pod2gen.Person`
        """

        if isinstance(person, Person):
            self.__persons.append(person)
        else:
            raise ValueError("%r is not a person object" % person)

    @property
    def persons(self):
        """A list of Person objects representing persons of interest to the episode.

        .. seealso::
            the method :meth:`Episode.add_person() <.add_person>` to add
            :class:`~.pod2gen.Person` objects

        :type: :obj:`list` of :class:`~pod2gen.Person`
        :RSS: ``<podcast:person>`` elements
        """
        return self.__persons

    @persons.setter
    def persons(self, persons):
        if persons is not None:
            try:
                persons = list(persons)
            except TypeError:
                raise ValueError("persons must be a list/tuple of Person objects")

            for person in persons:
                if not isinstance(person, Person):
                    raise ValueError("persons must be a list/tuple of Person objects")

            self.__persons = persons
        else:
            self.__persons = []



# -*- coding: utf-8 -*-
"""
    pod2gen
    ~~~~~~

    Package which makes it easy to generate podcast RSS using Python.

    See the official documentation at https://pod2gen.caproni.fm

    :copyright: 2016, Thorben Dahl <thorben@sjostrom.no>
        2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
from .alternate_media import AlternateMedia
from .category import Category
from .constants import EPISODE_TYPE_BONUS, EPISODE_TYPE_FULL, EPISODE_TYPE_TRAILER
from .episode import Episode
from .funding import Funding
from .license import License
from .location import Location
from .media import Media
from .person import Person
from .podcast import Podcast
from .soundbite import Soundbite
from .trailer import Trailer
from .transcript import Transcript
from .util import htmlencode
from .warnings import (
    LegacyCategoryWarning,
    NotRecommendedWarning,
    NotSupportedByItunesWarning,
    Pod2genWarning,
)



# -*- coding: utf-8 -*-
"""
    pod2gen.transcript
    ~~~~~~~~~~~~~~~

    This module contains Transcript, which represents an episode transcript.

    :copyright: 2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import warnings
from urllib.parse import urlparse

import pycountry
import validators

from pod2gen.warnings import NotSupportedByItunesWarning


class Transcript(object):
    """Class representing an Episode transcript object.

    By using this class, you can be sure that the transcript object is formatted correctly
    and will be correctly used by the associated :class:`~.pod2gen.Episode` object.

    :class:`.Transcript` objects can be attached to :class:`.Episode` objects
    using this method. Transcripts are stored in :attr:`.Episode.transcripts`.

    See `podcast-namespace Transcript
    <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#transcript>`_
    for an overview of the Transcript tag.

    Example::

        >>> from pod2gen import Transcript
        >>> t = Transcript("https://examples.com/transcript_sample.txt", "text/html", language="es", is_caption=False)
        >>> t.url
        'https://examples.com/transcript_sample.txt'
        >>> t.type
        'text/html'
        >>> t.language
        'es'
        >>> t.is_caption
        False
        >>>
        >>> t = Transcript("https://examples.com/transcript_sample_2.txt", "text/txt")
        >>> t.url
        'https://examples.com/transcript_sample_2.txt'
        >>> t.type
        'text/txt'
        >>> t.is_caption
        False
        >>> t.is_caption = True
        >>> t.is_caption
        True
        >>> my_episode.add_transcript(t)
    """

    _accepted_types = [
        "text/plain",
        "text/html",
        "text/vtt",
        "text/srt",
        "application/srt",
        "application/json",
    ]

    def __init__(self, url, type_, language=None, is_caption=False):
        """Create new Transcript object. See the class description of
        :class:´~pod2gen.transcript.Transcript`.

        :param url: Url of the transcript (Must be a valid url).
        :type url: str
        :param type: Mime type of the transcript file.
        :type type: str
        :param language: (Optional) Language of the transcript.
        :type language: str or None
        :param is_caption: (Optional) Set to True if the linked
        file is considered to be a closed captions file
        :type is_caption: bool
        """

        self._url = None
        self._type = None
        self._language = None
        self._is_caption = None

        self.url = url
        self.type = type_
        self.language = language
        self.is_caption = is_caption

    @property
    def url(self):
        """The URL at which this transcript is publicly accessible.

        Check `podcast-namespace Transcript url attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes>`_
        for more details.

        Only absolute URLs are allowed, so make sure it starts with http:// or
        https://. The server should support HEAD-requests and byte-range
        requests.

        Ensure you quote parts of the URL that are not supposed to carry any
        special meaning to the browser, typically the name of your file.
        Common offenders include the slash character when not used to separate
        folders, the hash mark (#) and the question mark (?).

        :type: :obj:`str`
        """
        return self._url

    @url.setter
    def url(self, url):
        if not url:
            raise ValueError("url cannot be empty or None")

        if validators.url(url) is not True:
            raise ValueError("Not a valid url: %s" % url)

        parsed_url = urlparse(url)
        if parsed_url.scheme not in ("http", "https"):
            warnings.warn(
                "URL scheme %s is not supported by iTunes. Make sure "
                "you use absolute URLs and HTTP or HTTPS." % parsed_url.scheme,
                NotSupportedByItunesWarning,
                stacklevel=2,
            )
        self._url = url

    @property
    def type(self):
        """The Mime type of the linked file :attr:`~.Transcript.url`.

        Check `podcast-namespace Transcript type attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes>`_
        for more details.

        :type: :obj:`str`
        """
        return self._type

    @type.setter
    def type(self, type_):
        if not type_:
            raise ValueError("Transcript type cannot be empty or None")

        type_ = type_.lower()
        if type_ not in self._accepted_types:
            raise ValueError(
                "Transcript type must be one of %r, received %r instead"
                % (self._accepted_types, type_)
            )

        self._type = type_

    @property
    def language(self):
        """An `IETF language tag (BCP 47) code <https://en.wikipedia.org/wiki/IETF_language_tag>`_
        identifying the language of the linked transcript.

        Check `podcast-namespace Transcript language attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes>`_
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self._language

    @language.setter
    def language(self, language):
        if language:
            language_country = language.split("-")
            language = language_country[0]
            if len(language_country) > 1:
                country = language_country[1]
            else:
                country = None

            try:
                language = pycountry.languages.get(alpha_2=language).alpha_2
            except:
                raise ValueError("Language code %r is unknown" % language)

            if country:
                try:
                    country = pycountry.countries.lookup(country).alpha_2
                    self._language = language + "-" + country
                except:
                    raise ValueError("Country code %r is unknown" % country)
            else:
                self._language = language

        else:
            self._language = language

    @property
    def is_caption(self):
        """If set to True, the linked file is considered to be a closed captions file,
        regardless of what the mime type is. In that scenario, time codes are assumed
        to be present in the file in some capacity.

        Check `podcast-namespace Transcript rel attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes>`_
        for more details.

        :type: :obj:`bool`
        """
        return self._is_caption

    @is_caption.setter
    def is_caption(self, is_caption):
        if not isinstance(is_caption, bool):
            raise ValueError(
                "is_caption must be a boolean. Received object of type %r instead"
                % type(is_caption)
            )
        self._is_caption = is_caption

    def __repr__(self):
        return "Transcript(url=%s, type=%s, language=%s, is_caption=%s)" % (
            self.url,
            self.type,
            self.language,
            self.is_caption,
        )



# -*- coding: utf-8 -*-
"""
    pod2gen.category
    ~~~~~~~~~~~~~~~

    This module contains Category, which represents a single iTunes category.

    :copyright: 2016, Thorben Dahl <thorben@sjostrom.no>
        2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import warnings

from pod2gen.warnings import LegacyCategoryWarning


class Category(object):
    """Immutable class representing an Apple Podcasts category.

    By using this class, you can be sure that the chosen category is a
    valid category, that it is formatted correctly and you will be warned
    when using an old category.

    See https://help.apple.com/itc/podcasts_connect/#/itc9267a2f12 for an
    overview of the available categories and their subcategories.

    .. versionchanged:: 1.1.0
       Updated to reflect `the new categories <https://podnews.net/article/apple-changed-podcast-categories-2019>`_
       as of August 9th 2019 and yield a
       :class:`~pod2gen.warnings.LegacyCategoryWarning` when using one of the
       old categories.

    .. note::

        The categories are case-insensitive, and you may escape ampersands.
        The category and subcategory will end up properly capitalized and
        with unescaped ampersands.

    Example::

        >>> from pod2gen import Category
        >>> c = Category("Music")
        >>> c.category
        'Music'
        >>> c.subcategory is None
        True
        >>>
        >>> d = Category("games &amp; hobbies", "Video games")
        >>> d.category
        'Games & Hobbies'
        >>> d.subcategory
        'Video Games'
    """

    _legacy_categories = {
        "Arts": [
            "Design",
            "Fashion & Beauty",
            "Food",
            "Literature",
            "Performing Arts",
            "Visual Arts",
        ],
        "Business": [
            "Business News",
            "Careers",
            "Investing",
            "Management & Marketing",
            "Shopping",
        ],
        "Comedy": [],
        "Education": [
            "Education",
            "Education Technology",
            "Higher Education",
            "K-12",
            "Language Courses",
            "Training",
        ],
        "Games & Hobbies": [
            "Automotive",
            "Aviation",
            "Hobbies",
            "Other Games",
            "Video Games",
        ],
        "Government & Organizations": ["Local", "National", "Non-Profit", "Regional"],
        "Health": [
            "Alternative Health",
            "Fitness & Nutrition",
            "Self-Help",
            "Sexuality",
        ],
        "Kids & Family": [],
        "Music": [],
        "News & Politics": [],
        "Religion & Spirituality": [
            "Buddhism",
            "Christianity",
            "Hinduism",
            "Islam",
            "Judaism",
            "Other",
            "Spirituality",
        ],
        "Science & Medicine": ["Medicine", "Natural Sciences", "Social Sciences"],
        "Society & Culture": [
            "History",
            "Personal Journals",
            "Philosophy",
            "Places & Travel",
        ],
        "Sports & Recreation": [
            "Amateur",
            "College & High School",
            "Outdoor",
            "Professional",
        ],
        "Technology": ["Gadgets", "Tech News", "Podcasting", "Software How-To"],
        "TV & Film": [],
    }

    _categories = {
        "Arts": [
            "Books",
            "Design",
            "Fashion & Beauty",
            "Food",
            "Performing Arts",
            "Visual Arts",
        ],
        "Business": [
            "Careers",
            "Entrepreneurship",
            "Investing",
            "Management",
            "Marketing",
            "Non-Profit",
        ],
        "Comedy": [
            "Comedy Interviews",
            "Improv",
            "Stand-up",
        ],
        "Education": [
            "Courses",
            "How To",
            "Language Learning",
            "Self-Improvement",
        ],
        "Fiction": [
            "Comedy Fiction",
            "Drama",
            "Science Fiction",
        ],
        "Government": [],
        "History": [],
        "Health & Fitness": [
            "Alternative Health",
            "Fitness",
            "Medicine",
            "Mental Health",
            "Nutrition",
            "Sexuality",
        ],
        "Kids & Family": [
            "Education for Kids",
            "Parenting",
            "Pets & Animals",
            "Stories for Kids",
        ],
        "Leisure": [
            "Animation & Manga",
            "Automotive",
            "Aviation",
            "Crafts",
            "Games",
            "Hobbies",
            "Home & Garden",
            "Video Games",
        ],
        "Music": [
            "Music Commentary",
            "Music History",
            "Music Interviews",
        ],
        "News": [
            "Business News",
            "Daily News",
            "Entertainment News",
            "News Commentary",
            "Politics",
            "Sports News",
            "Tech News",
        ],
        "Religion & Spirituality": [
            "Buddhism",
            "Christianity",
            "Hinduism",
            "Islam",
            "Judaism",
            "Religion",
            "Spirituality",
        ],
        "Science": [
            "Astronomy",
            "Chemistry",
            "Earth Sciences",
            "Life Sciences",
            "Mathematics",
            "Natural Sciences",
            "Nature",
            "Physics",
            "Social Sciences",
        ],
        "Society & Culture": [
            "Documentary",
            "Personal Journals",
            "Philosophy",
            "Places & Travel",
            "Relationships",
        ],
        "Sports": [
            "Baseball",
            "Basketball",
            "Cricket",
            "Fantasy Sports",
            "Football",
            "Golf",
            "Hockey",
            "Rugby",
            "Running",
            "Soccer",
            "Swimming",
            "Tennis",
            "Volleyball",
            "Wilderness",
            "Wrestling",
        ],
        "Technology": [],
        "True Crime": [],
        "TV & Film": [
            "After Shows",
            "Film History",
            "Film Interviews",
            "Film Reviews",
            "TV Reviews",
        ],
    }

    def __init__(self, category, subcategory=None):
        """Create new Category object. See the class description of
        :class:´~pod2gen.category.Category`.

        :param category: Category of the podcast.
        :type category: str
        :param subcategory: (Optional) Subcategory of the podcast.
        :type subcategory: str or None
        """
        if not category:
            raise TypeError('category must be provided, was "%s"' % category)
        try:
            canonical_category, canonical_subcategory = self._look_up_category(
                category,
                subcategory,
                self._categories,
            )
        except ValueError:
            # Maybe this is a legacy category?
            canonical_category, canonical_subcategory = self._look_up_category(
                category,
                subcategory,
                self._legacy_categories,
            )
            # Okay, it is, warn about this
            warnings.warn(
                'The category ("%s", "%s") is a legacy category. Please switch '
                "to one of the new Apple Podcast categories."
                % (canonical_category, canonical_subcategory),
                category=LegacyCategoryWarning,
                stacklevel=2,
            )

        self.__category = canonical_category
        self.__subcategory = canonical_subcategory

    def _look_up_category(self, category, subcategory, available_categories):
        # Do a case-insensitive search for the category
        search_category = category.strip().replace("&amp;", "&").lower()
        for actual_category in available_categories:
            if actual_category.lower() == search_category:
                # We found it
                canonical_category = actual_category
                break
        else:  # no break
            raise ValueError('Invalid category "%s"' % category)

        # Do a case-insensitive search for the subcategory, if provided
        canonical_subcategory = None
        if subcategory is not None:
            search_subcategory = subcategory.strip().replace("&amp;", "&").lower()
            for actual_subcategory in available_categories[canonical_category]:
                if actual_subcategory.lower() == search_subcategory:
                    canonical_subcategory = actual_subcategory
                    break
            else:  # no break
                raise ValueError(
                    'Invalid subcategory "%s" under category "%s"'
                    % (subcategory, canonical_category)
                )

        return canonical_category, canonical_subcategory

    @property
    def category(self):
        """The category represented by this object. Read-only.

        :type: :obj:`str`
        """
        return self.__category
        # Make this attribute read-only by not implementing setter

    @property
    def subcategory(self):
        """The subcategory this object represents. Read-only.

        :type: :obj:`str`
        """
        return self.__subcategory
        # Make this attribute read-only by not implementing setter

    def __repr__(self):
        return "Category(category=%s, subcategory=%s)" % (
            self.category,
            self.subcategory,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import Extension, setup

long_description = """
pod2gen
=======

This module can be used to easily generate Podcasts. It is designed so you
don't need to read up on how RSS and iTunes functions – it just works!

See the documentation at https://pod2gen.caproni.fm/en/latest/ for more
information.

It is licensed under the terms of both the FreeBSD license and the LGPLv3+.
Choose the one which is more convenient for you. For more details have a look
at license.bsd and license.lgpl."""

setup(
    name="pod2gen",
    packages=["pod2gen"],
    # Remember to update the version in pod2gen.version, too!
    version="1.0.3",
    description="Generating podcasts with Python should be easy!",
    author="Slim Beji",
    author_email="mslimbeji@gmail.com",
    url="https://pod2gen.caproni.fm/en/latest/",
    keywords=["feed", "RSS", "podcast", "iTunes", "generator"],
    license="FreeBSD and LGPLv3+",
    install_requires=[
        "lxml",
        "dateutils",
        "tinytag",
        "requests",
        "pycountry",
        "validators",
    ],
    python_requires=">=3.6.*",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Communications",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing",
        "Topic :: Text Processing :: Markup",
        "Topic :: Text Processing :: Markup :: XML",
    ],
    long_description_content_type="text/x-rst",
    long_description=long_description,
)



# -*- coding: utf-8 -*-
"""
    pod2gen.trailer
    ~~~~~~~~~~~~~~~

    This module contains Trailer, which represents a podcast trailer.

    :copyright: 2021, Slim Beji <mslimbeji@gmail.com>
    :license: FreeBSD and LGPL, see license.* for more details.
"""
import mimetypes
from datetime import datetime
from urllib.parse import urlparse

import validators
from lxml import etree

from pod2gen.util import formatRFC2822
from pod2gen.warnings import NotSupportedByItunesWarning


class Trailer(object):
    """Class representing a podcast Trailer object.

    By using this class, you can be sure that the soundbite object is formatted correctly
    and will be correctly used by the associated :class:`~.pod2gen.Podcast` object.

    :class:`.Trailer` objects can be attached to :class:`.Podcast` objects
    using this method. Trailers are stored in :attr:`.Podcast.trailers`.

    See `podcast-namespace Trailer
    <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#trailer>`_
    for an overview of the Trailer tag

    Example::

        >>> from pod2gen import Trailer, Podcast
        >>> from datetime import datetime
        >>> from dateutil import tz
        >>> pubdate = datetime(2021, 8, 15, 8, 15, 12, 0, tz.UTC)
        >>> p = Podcast()
        >>> t = Trailer("Coming April 1st, 2021", "https://example.org/trailers/teaser", pubdate)
        >>> t.text
        'Coming April 1st, 2021'
        >>> t.url
        'https://example.org/trailers/teaser'
        >>> t.pubdate
        datetime.datetime(2021, 8, 15, 8, 15, 12, tzinfo=<UTC>)
        >>> t.text = "Coming August 15th, 2021"
        >>> t.text
        'Coming August 15th, 2021'
        >>> t.length = 12345678
        >>> t.length
        12345678
        >>> t.type = "audio/mp3"
        >>> t.type
        'audio/mp3'
        >>> t.season = 2
        >>> t.season
        2
        >>> p.add_trailer(t)
        >>> t2 = Trailer("Another teaser", "https://example.org/trailers/teaser", pubdate, type="audio/mp3")
        >>> t2.type
        'audio/mp3'
        >>> p.add_trailer(t2)

    """

    def __init__(self, text, url, pubdate, length=None, type=None, season=None):
        """Create new Trailer object. See the class description of
        :class:´~pod2gen.trailer.Trailer`.

        :param text: The title of the trailer.
        :type text: str
        :param url: Link of the audio or video file to be played
        :type url: str
        :param pubdate: The date the trailer was published. Must be a timezone aware
            datetime object.
        :type pubdate: datetime.datetime
        :param length: (Optional) The length of the file in bytes.
        :type length: int
        :param type: (Optional) The mime type of the file.
        :type type: str
        :param season: (Optional) Specifies that this trailer is for a particular season number.
        :type season: int
        """

        self.__text = None
        self.__url = None
        self.__pubdate = None
        self.__length = None
        self.__type = None
        self.__season = None

        self.text = text
        self.url = url
        self.pubdate = pubdate
        self.length = length
        self.type = type
        self.season = season

        # Trying to guess the mime type from the url
        # if no explicit type was provided
        if self.type is None:
            try:
                self.type = mimetypes.guess_type(self.url)
            except:
                pass

    def rss_entry(self):
        """Create an RSS ``<podcast:trailer>`` tag using lxml's etree and return it.

        This is primarily used by :class:`~.pod2gen.Podcast` for building
        the RSS feed.

        :RSS: ``<podcast:location>``

        :returns: :class:`lxml.etree.Element`
        """

        PODCAST_NS = "https://podcastindex.org/namespace/1.0"
        entry = etree.Element("{%s}trailer" % PODCAST_NS)

        entry.text = self.text
        entry.attrib["url"] = self.url
        entry.attrib["pubdate"] = formatRFC2822(self.pubdate)

        if self.length:
            entry.attrib["length"] = str(self.length)

        if self.type:
            entry.attrib["type"] = self.type

        if self.season:
            entry.attrib["season"] = str(self.season)

        return entry

    @property
    def text(self):
        """The title of the trailer. It should not exceed 128 characters.

        Check `podcast-namespace Trailer node value
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#node-value-7>`_
        for more details.

        :type: :obj:`str`
        """
        return self.__text

    @text.setter
    def text(self, text):
        if not text:
            raise ValueError("Trailer text cannot be None or empty")

        if not isinstance(text, str):
            raise ValueError("Trailer text must be a valid string")

        if len(text) > 128:
            raise ValueError("Trailer text must not exceed 128 characters")

        self.__text = text

    @property
    def url(self):
        """The url/link that points to the audio or video file to be played.

        Check `podcast-namespace Trailer url attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-9>`_
        for more details.

        :type: :obj:`str`
        """
        return self.__url

    @url.setter
    def url(self, url):
        if not url:
            raise ValueError("url cannot be empty or None")

        if validators.url(url) is not True:
            raise ValueError("Not a valid url: %s" % url)

        parsed_url = urlparse(url)
        if parsed_url.scheme not in ("http", "https"):
            warnings.warn(
                "URL scheme %s is not supported by iTunes. Make sure "
                "you use absolute URLs and HTTP or HTTPS." % parsed_url.scheme,
                NotSupportedByItunesWarning,
                stacklevel=2,
            )
        self.__url = url

    @property
    def pubdate(self):
        """The date the trailer was published. It must a be timezone aware
        :obj:`datetime.datetime` object.
        Ideally use :mod:`dateutil.tz` module for handling timezones. Alternatively,
        :mod:`pytz` can also be used.

        Check `podcast-namespace Trailer pubdate attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-9>`_
        for more details.

        :type: :obj:`datetime.datetime`
        """
        return self.__pubdate

    @pubdate.setter
    def pubdate(self, pubdate):
        if pubdate is None:
            raise ValueError("pubdate cannot be None")

        if isinstance(pubdate, datetime):
            if not pubdate.tzinfo:
                raise ValueError("pubdate must have a defined timezone.")
        else:
            raise TypeError("pubdate must be a timezone aware datetime")

        self.__pubdate = pubdate

    @property
    def length(self):
        """The length of the trailer audio/video file in bytes.

        Check `podcast-namespace Trailer length attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-9>`_
        for more details.

        :type: :obj:`int` or :obj:`None`
        """
        return self.__length

    @length.setter
    def length(self, length):
        if length:
            try:
                length = int(length)
            except ValueError:
                raise TypeError(
                    "length must be an integer. %r could not be converted into integer."
                    % length
                )

            self.__length = length
        else:
            self.__length = None

    @property
    def type(self):
        """The mime type of the trailer file.

        Check `podcast-namespace Trailer type attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-9>`_
        for more details.

        :type: :obj:`str` or :obj:`None`
        """
        return self.__type

    @type.setter
    def type(self, type):
        if type:
            if not isinstance(type, str):
                raise TypeError("File mime type must be a valid string")
            self.__type = type

        else:
            self.__type = None

    @property
    def season(self):
        """If this attribute is present it specifies that the trailer is for a particular
        season number.

        Check `podcast-namespace Trailer season attribute
        <https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md#attributes-9>`_
        for more details.

        :type: :obj:`int` or :obj:`None`
        """
        return self.__season

    @season.setter
    def season(self, season):
        if season:
            try:
                season = int(season)
            except ValueError:
                raise TypeError(
                    "season must be an integer. %r could not be converted into integer."
                    % season
                )

            self.__season = season
        else:
            self.__season = None

    def __repr__(self):
        kwargs = {
            "text": self.text,
            "url": self.url,
            "pubdate": self.pubdate,
            "length": self.length,
            "type": self.type,
            "season": self.season,
        }

        output = ""
        for k, v in kwargs.items():
            if v:
                if k == "pubdate":
                    v = formatRFC2822(self.pubdate)

                output = "%s%s='%s', " % (output, k, str(v))

        return "Trailer(%s)" % output[:-2]



# -*- coding: utf-8 -*-
"""
    pod2gen.version
    ~~~~~~~~~~~~~~~

    :copyright: 2013, Lars Kiesow <lkiesow@uos.de>; 2016, Thorben Dahl
        <thorben@sjostrom.no>; 2021, Slim Beji <mslimbeji@gmail.com>

    :license: FreeBSD and LGPL, see license.* for more details.

"""
"Version of pod2gen represented as tuple"
version = (1, 0, 3)


"Version of pod2gen represented as string"
version_str = ".".join([str(x) for x in version])

version_major = version[:1]
version_minor = version[:2]
version_full = version

version_major_str = ".".join([str(x) for x in version_major])
version_minor_str = ".".join([str(x) for x in version_minor])
version_full_str = ".".join([str(x) for x in version_full])

"Name of this project"
name = "pod2gen"

"Website of this project"
website = "https://pod2gen.caproni.fm"
